# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__CLIP_VOLUME_METADATA = Metadata(
    id="7bfb440cf4d5cd2d5ca3418b3dc2ad19927c0963.boutiques",
    name="@clip_volume",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VClipVolumeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__clip_volume(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_clipped_volume: OutputPathType | None
    """Output clipped or cropped volume"""
    output_followers: OutputPathType | None
    """Output for follower datasets after clipping/cropping"""


def v__clip_volume(
    input_volume: InputPathType,
    below_zmm: float | None = None,
    above_zmm: float | None = None,
    left_xmm: float | None = None,
    right_xmm: float | None = None,
    anterior_ymm: float | None = None,
    posterior_ymm: float | None = None,
    box: list[float] | None = None,
    mask_box: list[float] | None = None,
    and_logic: bool = False,
    or_logic: bool = False,
    verbosity: bool = False,
    crop_allzero: bool = False,
    crop_greedy: bool = False,
    crop: bool = False,
    crop_npad: float | None = None,
    output_prefix: str | None = None,
    followers: list[InputPathType] | None = None,
    runner: Runner | None = None,
) -> VClipVolumeOutputs:
    """
    A tool to clip regions of a volume in various ways, such as above/below certain
    coordinates or within a specified box.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        input_volume: Volume to clip.
        below_zmm: Set to 0 slices below Zmm.
        above_zmm: Set to 0 slices above Zmm.
        left_xmm: Set to 0 slices left of Xmm.
        right_xmm: Set to 0 slices right of Xmm.
        anterior_ymm: Set to 0 slices anterior to Ymm.
        posterior_ymm: Set to 0 slices posterior to Ymm.
        box: Clip the volume to a box centered at Cx, Cy, Cz (RAI mm), and of\
            dimensions Dx Dy Dz (RAI mm).
        mask_box: Set all values inside the box to 1. Box centered at Cx, Cy,\
            Cz (RAI mm), and of dimensions Dx Dy Dz (RAI mm).
        and_logic: Combine with next clipping planes using 'and'.
        or_logic: Combine with next clipping planes using 'or'.
        verbosity: Show command details (verbose output).
        crop_allzero: Crop the output volume with 3dAutobox -noclust.
        crop_greedy: Crop the output volume with 3dAutobox.
        crop: Same as -crop_greedy, kept for backward compatibility.
        crop_npad: Set 3dAutobox's -npad option to NPAD. NPAD fattens the\
            volume a little after cropping.
        output_prefix: Output prefix for the resultant volume. Default is the\
            input prefix with _clp suffixed to it.
        followers: Apply the same clipping or cropping treatment to the\
            follower datasets.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VClipVolumeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__CLIP_VOLUME_METADATA)
    cargs = []
    cargs.append("@clip_volume")
    cargs.append(execution.input_file(input_volume))
    if below_zmm is not None:
        cargs.extend([
            "-below",
            str(below_zmm)
        ])
    if above_zmm is not None:
        cargs.extend([
            "-above",
            str(above_zmm)
        ])
    if left_xmm is not None:
        cargs.extend([
            "-left",
            str(left_xmm)
        ])
    if right_xmm is not None:
        cargs.extend([
            "-right",
            str(right_xmm)
        ])
    if anterior_ymm is not None:
        cargs.extend([
            "-anterior",
            str(anterior_ymm)
        ])
    if posterior_ymm is not None:
        cargs.extend([
            "-posterior",
            str(posterior_ymm)
        ])
    if box is not None:
        cargs.extend([
            "-box",
            *map(str, box)
        ])
    if mask_box is not None:
        cargs.extend([
            "-mask_box",
            *map(str, mask_box)
        ])
    if and_logic:
        cargs.append("-and")
    if or_logic:
        cargs.append("-or")
    if verbosity:
        cargs.append("-verb")
    if crop_allzero:
        cargs.append("-crop_allzero")
    if crop_greedy:
        cargs.append("-crop_greedy")
    if crop:
        cargs.append("-crop")
    if crop_npad is not None:
        cargs.extend([
            "-crop_npad",
            str(crop_npad)
        ])
    if output_prefix is not None:
        cargs.extend([
            "-prefix",
            output_prefix
        ])
    if followers is not None:
        cargs.extend([
            "-followers",
            *[execution.input_file(f) for f in followers]
        ])
    ret = VClipVolumeOutputs(
        root=execution.output_file("."),
        output_clipped_volume=execution.output_file(output_prefix + "_clp.nii.gz") if (output_prefix is not None) else None,
        output_followers=execution.output_file(output_prefix + "_follow_clp.nii.gz") if (output_prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VClipVolumeOutputs",
    "V__CLIP_VOLUME_METADATA",
    "v__clip_volume",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_XVOLAVG_METADATA = Metadata(
    id="ef5784badf94298a73c9e048e6ca4f281cd306da.boutiques",
    name="mri_xvolavg",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriXvolavgOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_xvolavg(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    averaged_output: OutputPathType
    """The averaged output volume."""


def mri_xvolavg(
    input_volumes: list[InputPathType],
    vol_type: str,
    output_volume: str,
    output_type: str | None = None,
    runner: Runner | None = None,
) -> MriXvolavgOutputs:
    """
    Tool to average multiple volumes together (including 4D volumes).
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_volumes: Path(s) to input volume(s). This option can be repeated\
            for each input volume.
        vol_type: Format type of all input volumes.
        output_volume: Path to output volume.
        output_type: Format type of the output volume (default is that of input\
            volumes).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriXvolavgOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_XVOLAVG_METADATA)
    cargs = []
    cargs.append("mri_xvolavg")
    cargs.extend([
        "--vol",
        *[execution.input_file(f) for f in input_volumes]
    ])
    cargs.extend([
        "--vol_type",
        vol_type
    ])
    cargs.extend([
        "--out",
        output_volume
    ])
    if output_type is not None:
        cargs.extend([
            "--out_type",
            output_type
        ])
    ret = MriXvolavgOutputs(
        root=execution.output_file("."),
        averaged_output=execution.output_file(output_volume),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_XVOLAVG_METADATA",
    "MriXvolavgOutputs",
    "mri_xvolavg",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SEG2FILLED_METADATA = Metadata(
    id="894fcea3b0a2a4a1b34ad187f3a8016973d50857.boutiques",
    name="seg2filled",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class Seg2filledOutputs(typing.NamedTuple):
    """
    Output object returned when calling `seg2filled(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    out_filled_mgz: OutputPathType
    """The output filled MGZ file."""


def seg2filled(
    seg_file: InputPathType,
    norm_file: InputPathType,
    output_file: str,
    ndil: int | None = None,
    cavity_flag: bool = False,
    surf_name: str | None = None,
    surf_dir: str | None = None,
    runner: Runner | None = None,
) -> Seg2filledOutputs:
    """
    Creates a filled.mgz from an aseg-style segmentation using SAMSEG segmentation.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        seg_file: Input aseg-style segmentation file (e.g., seg.mgz).
        norm_file: Normalization file (e.g., norm.mgz).
        output_file: Filled output file (e.g., filled.mgz).
        ndil: Number of iterations for dilation used to speed cavity detection.
        cavity_flag: Simulate a cavity to test the filling operation.
        surf_name: Name for the surface output (creates ?h.surfname).
        surf_dir: Directory to put the surface (default is same as filled).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Seg2filledOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SEG2FILLED_METADATA)
    cargs = []
    cargs.append("seg2filled")
    cargs.extend([
        "-seg",
        "-" + execution.input_file(seg_file)
    ])
    cargs.extend([
        "-norm",
        "-" + execution.input_file(norm_file)
    ])
    cargs.extend([
        "-o",
        "-" + output_file
    ])
    if ndil is not None:
        cargs.extend([
            "--ndil",
            str(ndil)
        ])
    if cavity_flag:
        cargs.append("--cavity")
    if surf_name is not None:
        cargs.extend([
            "--surf",
            surf_name
        ])
    if surf_dir is not None:
        cargs.extend([
            "--surfdir",
            surf_dir
        ])
    ret = Seg2filledOutputs(
        root=execution.output_file("."),
        out_filled_mgz=execution.output_file(output_file),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SEG2FILLED_METADATA",
    "Seg2filledOutputs",
    "seg2filled",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_RESCALE_METADATA = Metadata(
    id="dc82fd3ed28ce8af91d50743abd00b226c549b2e.boutiques",
    name="mris_rescale",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisRescaleOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_rescale(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    rescaled_output_surface: OutputPathType
    """The file containing the rescaled surface."""


def mris_rescale(
    input_surface: InputPathType,
    output_surface: str,
    runner: Runner | None = None,
) -> MrisRescaleOutputs:
    """
    This program will rescale a surface representation.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_surface: The input surface file to be rescaled.
        output_surface: The output surface file after rescaling.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisRescaleOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_RESCALE_METADATA)
    cargs = []
    cargs.append("mris_rescale")
    cargs.append(execution.input_file(input_surface))
    cargs.append(output_surface)
    ret = MrisRescaleOutputs(
        root=execution.output_file("."),
        rescaled_output_surface=execution.output_file(output_surface),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_RESCALE_METADATA",
    "MrisRescaleOutputs",
    "mris_rescale",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SURFACE_CURVATURE_METADATA = Metadata(
    id="53a3b6f741f93ffb1d3ca64e3b70f76507df8aa9.boutiques",
    name="SurfaceCurvature",
    package="ants",
    container_image_tag="antsx/ants:v2.5.3",
)


class SurfaceCurvatureOutputs(typing.NamedTuple):
    """
    Output object returned when calling `surface_curvature(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image: OutputPathType
    """The processed image file."""


def surface_curvature(
    filename_in: InputPathType,
    filename_out: InputPathType,
    sigma: float,
    option: float,
    runner: Runner | None = None,
) -> SurfaceCurvatureOutputs:
    """
    The Shape Operator for Differential Analysis of Images. It can operate on binary
    or gray scale images with various modes to see different effects.
    
    Author: ANTs Developers
    
    URL: https://github.com/ANTsX/ANTs
    
    Args:
        filename_in: The input image file in .nii format.
        filename_out: The output image file in .nii format.
        sigma: The sigma value for analysis.
        option: The operation mode: 0 for mean curvature, 5 for surface\
            characterization, 6 for Gaussian curvature, and 7 for surface area.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceCurvatureOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_CURVATURE_METADATA)
    cargs = []
    cargs.append("SurfaceCurvature" + execution.input_file(filename_in) + execution.input_file(filename_out) + str(sigma) + str(option))
    ret = SurfaceCurvatureOutputs(
        root=execution.output_file("."),
        output_image=execution.output_file(pathlib.Path(filename_out).name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SURFACE_CURVATURE_METADATA",
    "SurfaceCurvatureOutputs",
    "surface_curvature",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_REFINE_SEG_METADATA = Metadata(
    id="e3e44731d7475ba42bf997ea5583c4a788afb781.boutiques",
    name="mri_refine_seg",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriRefineSegOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_refine_seg(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    refined_output: OutputPathType
    """Refined segmentation output file."""


def mri_refine_seg(
    input_segmentation: InputPathType,
    output_segmentation: InputPathType,
    debug: bool = False,
    runner: Runner | None = None,
) -> MriRefineSegOutputs:
    """
    Refines a messy segmentation by recoding stray voxels.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_segmentation: Input segmentation file.
        output_segmentation: Refined segmentation output name.
        debug: Save the original segmentation, a mask of the refined voxels,\
            and a pointset of the refined clusters to the output directory.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriRefineSegOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_REFINE_SEG_METADATA)
    cargs = []
    cargs.append("mri_refine_seg")
    cargs.extend([
        "-i",
        execution.input_file(input_segmentation)
    ])
    cargs.extend([
        "-o",
        execution.input_file(output_segmentation)
    ])
    if debug:
        cargs.append("--debug")
    ret = MriRefineSegOutputs(
        root=execution.output_file("."),
        refined_output=execution.output_file(pathlib.Path(output_segmentation).name),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_REFINE_SEG_METADATA",
    "MriRefineSegOutputs",
    "mri_refine_seg",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

REG_RESAMPLE_METADATA = Metadata(
    id="2ccb8532ef070b252ef142cb969c94469f490a69.boutiques",
    name="reg_resample",
    package="niftyreg",
    container_image_tag="vnmd/niftyreg_1.4.0:20220819",
)


class RegResampleOutputs(typing.NamedTuple):
    """
    Output object returned when calling `reg_resample(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_resampled_image: OutputPathType | None
    """File containing the resampled image"""
    output_resampled_blank: OutputPathType | None
    """File containing the resampled blank grid"""


def reg_resample(
    reference_image: InputPathType,
    floating_image: InputPathType,
    affine_transform: InputPathType | None = None,
    flirt_affine_transform: InputPathType | None = None,
    control_point_grid: InputPathType | None = None,
    deformation_field: InputPathType | None = None,
    resampled_image: str | None = None,
    resampled_blank: str | None = None,
    nearest_neighbor: bool = False,
    linear_interpolation: bool = False,
    runner: Runner | None = None,
) -> RegResampleOutputs:
    """
    Tool for resampling floating images to the reference image space using different
    transformations.
    
    Author: NiftyReg Developers
    
    URL: http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftyReg
    
    Args:
        reference_image: Filename of the reference image.
        floating_image: Filename of the floating image.
        affine_transform: Filename which contains an affine transformation\
            (Affine*Reference=Floating).
        flirt_affine_transform: Filename which contains a radiological flirt\
            affine transformation.
        control_point_grid: Filename of the control point grid image (from\
            reg_f3d).
        deformation_field: Filename of the deformation field image (from\
            reg_transform).
        resampled_image: Filename of the resampled image.
        resampled_blank: Filename of the resampled blank grid.
        nearest_neighbor: Use a Nearest Neighbor interpolation for the source\
            resampling (cubic spline by default).
        linear_interpolation: Use a Linear interpolation for the source\
            resampling (cubic spline by default).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `RegResampleOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(REG_RESAMPLE_METADATA)
    cargs = []
    cargs.append("reg_resample")
    cargs.extend([
        "-ref",
        execution.input_file(reference_image)
    ])
    cargs.extend([
        "-flo",
        execution.input_file(floating_image)
    ])
    if affine_transform is not None:
        cargs.extend([
            "-aff",
            execution.input_file(affine_transform)
        ])
    if flirt_affine_transform is not None:
        cargs.extend([
            "-affFlirt",
            execution.input_file(flirt_affine_transform)
        ])
    if control_point_grid is not None:
        cargs.extend([
            "-cpp",
            execution.input_file(control_point_grid)
        ])
    if deformation_field is not None:
        cargs.extend([
            "-def",
            execution.input_file(deformation_field)
        ])
    if resampled_image is not None:
        cargs.extend([
            "-res",
            resampled_image
        ])
    if resampled_blank is not None:
        cargs.extend([
            "-blank",
            resampled_blank
        ])
    if nearest_neighbor:
        cargs.append("-NN")
    if linear_interpolation:
        cargs.append("-LIN")
    ret = RegResampleOutputs(
        root=execution.output_file("."),
        output_resampled_image=execution.output_file(resampled_image) if (resampled_image is not None) else None,
        output_resampled_blank=execution.output_file(resampled_blank) if (resampled_blank is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "REG_RESAMPLE_METADATA",
    "RegResampleOutputs",
    "reg_resample",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_COMPUTE_PARC_OVERLAP_METADATA = Metadata(
    id="d598ee06b2c8bac2b0e66dc28fd526ceb983bfbd.boutiques",
    name="mris_compute_parc_overlap",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisComputeParcOverlapOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_compute_parc_overlap(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mris_compute_parc_overlap(
    subject: str,
    hemi: str,
    annot1: InputPathType | None = None,
    annot2: InputPathType | None = None,
    label1: InputPathType | None = None,
    label2: InputPathType | None = None,
    subj_dir: str | None = None,
    log_file: str | None = None,
    label_list: InputPathType | None = None,
    nocheck_label1_xyz: bool = False,
    nocheck_label2_xyz: bool = False,
    nocheck_label_xyz: bool = False,
    use_label1_xyz: bool = False,
    use_label2_xyz: bool = False,
    use_label_xyz: bool = False,
    debug_overlap: bool = False,
    runner: Runner | None = None,
) -> MrisComputeParcOverlapOutputs:
    """
    Compares two parcellated (annotated or labeled) surfaces and computes an overall
    Dice coefficient and mean minimum distances (mm).
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject: Subject to check.
        hemi: Hemisphere: rh or lh.
        annot1: First .annot file.
        annot2: Second .annot file.
        label1: First .label file.
        label2: Second .label file.
        subj_dir: Set SUBJECTS_DIR.
        log_file: Output the overall DICE and minimum distance to filename.
        label_list: File containing labels to check, one per line.
        nocheck_label1_xyz: When loading label1 file, don't check x,y,z coords\
            to surface.
        nocheck_label2_xyz: When loading label2 file, don't check x,y,z coords\
            to surface.
        nocheck_label_xyz: Do not check label1 and label2.
        use_label1_xyz: Replace surface x,y,z coords with those in label1 file.
        use_label2_xyz: Replace surface x,y,z coords with those in label2 file.
        use_label_xyz: Use label1 and label2 coords.
        debug_overlap: Generate ?h.overlap.annot.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisComputeParcOverlapOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_COMPUTE_PARC_OVERLAP_METADATA)
    cargs = []
    cargs.append("mris_compute_parc_overlap")
    cargs.extend([
        "--s",
        subject
    ])
    cargs.extend([
        "--hemi",
        hemi
    ])
    if annot1 is not None:
        cargs.extend([
            "--annot1",
            execution.input_file(annot1)
        ])
    if annot2 is not None:
        cargs.extend([
            "--annot2",
            execution.input_file(annot2)
        ])
    if label1 is not None:
        cargs.extend([
            "--label1",
            execution.input_file(label1)
        ])
    if label2 is not None:
        cargs.extend([
            "--label2",
            execution.input_file(label2)
        ])
    if subj_dir is not None:
        cargs.extend([
            "--sd",
            subj_dir
        ])
    if log_file is not None:
        cargs.extend([
            "--log",
            log_file
        ])
    if label_list is not None:
        cargs.extend([
            "--label-list",
            execution.input_file(label_list)
        ])
    if nocheck_label1_xyz:
        cargs.append("--nocheck-label1-xyz")
    if nocheck_label2_xyz:
        cargs.append("--nocheck-label2-xyz")
    if nocheck_label_xyz:
        cargs.append("--nocheck-label-xyz")
    if use_label1_xyz:
        cargs.append("--use-label1-xyz")
    if use_label2_xyz:
        cargs.append("--use-label2-xyz")
    if use_label_xyz:
        cargs.append("--use-label-xyz")
    if debug_overlap:
        cargs.append("--debug-overlap")
    ret = MrisComputeParcOverlapOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_COMPUTE_PARC_OVERLAP_METADATA",
    "MrisComputeParcOverlapOutputs",
    "mris_compute_parc_overlap",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

PROBTRACKX_METADATA = Metadata(
    id="053d5bf02cc2ecddcdfc27a671338716940fbabe.boutiques",
    name="probtrackx",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class ProbtrackxOutputs(typing.NamedTuple):
    """
    Output object returned when calling `probtrackx(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def probtrackx(
    samples: InputPathType,
    mask: InputPathType,
    seed: InputPathType,
    out: str = "fdt_paths",
    verbose: int | None = None,
    targetmasks: InputPathType | None = None,
    mask2: InputPathType | None = None,
    waypoints: InputPathType | None = None,
    network: bool = False,
    mesh: InputPathType | None = None,
    seedref: InputPathType | None = None,
    dir_: str | None = "logdir",
    forcedir: bool = False,
    opd: bool = False,
    pd: bool = False,
    os2t: bool = False,
    avoid: InputPathType | None = None,
    stop: InputPathType | None = None,
    xfm: InputPathType | None = None,
    invxfm: InputPathType | None = None,
    nsamples: int | None = 5000,
    nsteps: int | None = 2000,
    distthresh: float | None = 0,
    cthr: float | None = 0.2,
    fibthresh: float | None = 0.01,
    sampvox: bool = False,
    steplength: float | None = 0.5,
    loopcheck: bool = False,
    usef: bool = False,
    randfib: int | None = 0,
    fibst: int | None = 1,
    modeuler: bool = False,
    rseed: int | None = None,
    s2tastext: bool = False,
    runner: Runner | None = None,
) -> ProbtrackxOutputs:
    """
    Streamlines tracking algorithm for probabilistic tractography.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        samples: Basename for samples files.
        mask: Bet binary mask file in diffusion space.
        seed: Seed volume, or voxel, or ascii file with multiple volumes, or\
            freesurfer label file.
        out: Output file (default='fdt_paths').
        verbose: Verbose level, [0-2].
        targetmasks: File containing a list of target masks - required for\
            seeds_to_targets classification.
        mask2: Second mask in twomask_symm mode.
        waypoints: Waypoint mask or ascii list of waypoint masks - only keep\
            paths going through ALL the masks.
        network: Activate network mode - only keep paths going through at least\
            one seed mask (required if multiple seed masks).
        mesh: Freesurfer-type surface descriptor (in ascii format).
        seedref: Reference vol to define seed space in simple mode - diffusion\
            space assumed if absent.
        dir_: Directory to put the final volumes in - code makes this directory\
            - default='logdir'.
        forcedir: Use the actual directory name given - i.e. don't add + to\
            make a new directory.
        opd: Output path distribution.
        pd: Correct path distribution for the length of the pathways.
        os2t: Output seeds to targets.
        avoid: Reject pathways passing through locations given by this mask.
        stop: Stop tracking at locations given by this mask file.
        xfm: Transform taking seed space to DTI space (either FLIRT matrix or\
            FNIRT warpfield) - default is identity.
        invxfm: Transform taking DTI space to seed space (compulsory when using\
            a warpfield for seeds_to_dti).
        nsamples: Number of samples - default=5000.
        nsteps: Number of steps per sample - default=2000.
        distthresh: Discards samples shorter than this threshold (in mm -\
            default=0).
        cthr: Curvature threshold - default=0.2.
        fibthresh: Volume fraction before subsidary fibre orientations are\
            considered - default=0.01.
        sampvox: Sample random points within seed voxels.
        steplength: Steplength in mm - default=0.5.
        loopcheck: Perform loopchecks on paths - slower, but allows lower\
            curvature threshold.
        usef: Use anisotropy to constrain tracking.
        randfib: Default 0. Set to 1 to randomly sample initial fibres (with f\
            > fibthresh). Set to 2 to sample in proportion fibres (with\
            f>fibthresh) to f. Set to 3 to sample ALL populations at random (even\
            if f<fibthresh).
        fibst: Force a starting fibre for tracking - default=1, i.e. first\
            fibre orientation. Only works if randfib==0.
        modeuler: Use modified euler streamlining.
        rseed: Random seed.
        s2tastext: Output seed-to-target counts as a text file (useful when\
            seeding from a mesh).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `ProbtrackxOutputs`).
    """
    if verbose is not None and not (0 <= verbose <= 2): 
        raise ValueError(f"'verbose' must be between 0 <= x <= 2 but was {verbose}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(PROBTRACKX_METADATA)
    cargs = []
    cargs.append("probtrackx")
    cargs.extend([
        "-s",
        execution.input_file(samples)
    ])
    cargs.extend([
        "-m",
        execution.input_file(mask)
    ])
    cargs.extend([
        "-x",
        execution.input_file(seed)
    ])
    cargs.extend([
        "-o",
        out
    ])
    if verbose is not None:
        cargs.extend([
            "--verbose",
            str(verbose)
        ])
    if targetmasks is not None:
        cargs.extend([
            "--targetmasks",
            execution.input_file(targetmasks)
        ])
    if mask2 is not None:
        cargs.extend([
            "--mask2",
            execution.input_file(mask2)
        ])
    if waypoints is not None:
        cargs.extend([
            "--waypoints",
            execution.input_file(waypoints)
        ])
    if network:
        cargs.append("--network")
    if mesh is not None:
        cargs.extend([
            "--mesh",
            execution.input_file(mesh)
        ])
    if seedref is not None:
        cargs.extend([
            "--seedref",
            execution.input_file(seedref)
        ])
    if dir_ is not None:
        cargs.extend([
            "--dir",
            dir_
        ])
    if forcedir:
        cargs.append("--forcedir")
    if opd:
        cargs.append("--opd")
    if pd:
        cargs.append("--pd")
    if os2t:
        cargs.append("--os2t")
    if avoid is not None:
        cargs.extend([
            "--avoid",
            execution.input_file(avoid)
        ])
    if stop is not None:
        cargs.extend([
            "--stop",
            execution.input_file(stop)
        ])
    if xfm is not None:
        cargs.extend([
            "--xfm",
            execution.input_file(xfm)
        ])
    if invxfm is not None:
        cargs.extend([
            "--invxfm",
            execution.input_file(invxfm)
        ])
    if nsamples is not None:
        cargs.extend([
            "-P",
            str(nsamples)
        ])
    if nsteps is not None:
        cargs.extend([
            "-S",
            str(nsteps)
        ])
    if distthresh is not None:
        cargs.extend([
            "--distthresh",
            str(distthresh)
        ])
    if cthr is not None:
        cargs.extend([
            "-c",
            str(cthr)
        ])
    if fibthresh is not None:
        cargs.extend([
            "--fibthresh",
            str(fibthresh)
        ])
    if sampvox:
        cargs.append("--sampvox")
    if steplength is not None:
        cargs.extend([
            "--steplength",
            str(steplength)
        ])
    if loopcheck:
        cargs.append("-l")
    if usef:
        cargs.append("-f")
    if randfib is not None:
        cargs.extend([
            "--randfib",
            str(randfib)
        ])
    if fibst is not None:
        cargs.extend([
            "--fibst",
            str(fibst)
        ])
    if modeuler:
        cargs.append("--modeuler")
    if rseed is not None:
        cargs.extend([
            "--rseed",
            str(rseed)
        ])
    if s2tastext:
        cargs.append("--s2tastext")
    ret = ProbtrackxOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "PROBTRACKX_METADATA",
    "ProbtrackxOutputs",
    "probtrackx",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_AND_METADATA = Metadata(
    id="fc9c157fa97aee9cd814998f22d0f65be168bad4.boutiques",
    name="mri_and",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriAndOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_and(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def mri_and(
    input_files: list[InputPathType],
    runner: Runner | None = None,
) -> MriAndOutputs:
    """
    Performs a logical voxel-wise AND on a series of volumes.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_files: Input volume files.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriAndOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_AND_METADATA)
    cargs = []
    cargs.append("mri_and")
    cargs.extend([execution.input_file(f) for f in input_files])
    ret = MriAndOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_AND_METADATA",
    "MriAndOutputs",
    "mri_and",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_REFINE_SURFACES_METADATA = Metadata(
    id="afa5fdcf50348032d468a3e07292675e06d5374e.boutiques",
    name="mris_refine_surfaces",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisRefineSurfacesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_refine_surfaces(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    pial_surface: OutputPathType
    """Refined pial surface for the specified region"""
    white_surface: OutputPathType
    """Refined white matter surface for the specified region"""


def mris_refine_surfaces(
    subject_name: str,
    hemi: str,
    hires_volume: str,
    label_file: str,
    low_to_hires_xfm: str | None = None,
    sdir: str | None = None,
    use_mgz: bool = False,
    suffix: str | None = None,
    runner: Runner | None = None,
) -> MrisRefineSurfacesOutputs:
    """
    Refines cortical surfaces around the region specified by the label file.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subject_name: The name of the subject.
        hemi: The hemisphere to process ('lh' for left hemisphere, 'rh' for\
            right hemisphere).
        hires_volume: The high-resolution volume filename.
        label_file: The label file specifying the region to refine.
        low_to_hires_xfm: The optional low to high resolution transform file.
        sdir: Specify the SUBJECTS_DIR.
        use_mgz: Use .mgz volumes.
        suffix: Add specified suffix to the final surfaces.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisRefineSurfacesOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_REFINE_SURFACES_METADATA)
    cargs = []
    cargs.append("mris_refine_surfaces")
    cargs.append(subject_name)
    cargs.append(hemi)
    cargs.append(hires_volume)
    cargs.append(label_file)
    if low_to_hires_xfm is not None:
        cargs.append(low_to_hires_xfm)
    if sdir is not None:
        cargs.extend([
            "-sdir",
            sdir
        ])
    if use_mgz:
        cargs.append("-mgz")
    if suffix is not None:
        cargs.extend([
            "-suffix",
            suffix
        ])
    ret = MrisRefineSurfacesOutputs(
        root=execution.output_file("."),
        pial_surface=execution.output_file("$(SUBJECTS_DIR)/" + subject_name + "/surf/" + hemi + ".pialhires"),
        white_surface=execution.output_file("$(SUBJECTS_DIR)/" + subject_name + "/surf/" + hemi + ".whitehires"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_REFINE_SURFACES_METADATA",
    "MrisRefineSurfacesOutputs",
    "mris_refine_surfaces",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

VOL2SEGAVG_METADATA = Metadata(
    id="9d9d31cfe51a606166834e1114b170012714797f.boutiques",
    name="vol2segavg",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class Vol2segavgOutputs(typing.NamedTuple):
    """
    Output object returned when calling `vol2segavg(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_file: OutputPathType
    """Output file containing the computed average"""


def vol2segavg(
    output_file: str,
    input_volume: InputPathType,
    registration: str,
    segmentation_file: InputPathType,
    aparc_aseg_flag: bool = False,
    subject_id: str | None = None,
    segmentation_id: list[float] | None = None,
    multiply_value: float | None = None,
    no_bb_flag: bool = False,
    erode_value: float | None = None,
    dilate_value: float | None = None,
    wm_flag: bool = False,
    vcsf_flag: bool = False,
    xcsf_flag: bool = False,
    remove_mean_flag: bool = False,
    runner: Runner | None = None,
) -> Vol2segavgOutputs:
    """
    Computes the average of a volume inside a given segment of a segmentation
    resampling the input volume to the segmentation space.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        output_file: Output file, can be .txt or a binary (e.g., .nii, .mgh).
        input_volume: Input volume file (e.g., vol.nii).
        registration: Registration file, can be reg.dat or use --regheader.
        segmentation_file: Segmentation file (e.g., seg.mgz).
        aparc_aseg_flag: Use aparc+aseg flag.
        subject_id: Subject ID, may be needed if --reg not supplied.
        segmentation_id: Segmentation ID(s). Multiple IDs can be supplied.
        multiply_value: Multiply input by MulVal.
        no_bb_flag: Do not use bounding box.
        erode_value: Erode segmentation.
        dilate_value: Dilate segmentation.
        wm_flag: Sets segid to 2, 41, 7, 46, 251, 252, 253, 254, 255, 77, 78,\
            79 and use aparc+aseg.
        vcsf_flag: Sets segid to 4, 5, 43, 44, 31, 63 and use aparc+aseg.
        xcsf_flag: Sets segid to 257 and use apas+head.
        remove_mean_flag: Remove mean from time course.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `Vol2segavgOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOL2SEGAVG_METADATA)
    cargs = []
    cargs.append("vol2segavg")
    cargs.extend([
        "--o",
        output_file
    ])
    cargs.extend([
        "--i",
        execution.input_file(input_volume)
    ])
    cargs.extend([
        "--reg",
        registration
    ])
    cargs.extend([
        "--seg",
        execution.input_file(segmentation_file)
    ])
    if aparc_aseg_flag:
        cargs.append("--aparc+aseg")
    if subject_id is not None:
        cargs.extend([
            "--s",
            subject_id
        ])
    if segmentation_id is not None:
        cargs.extend([
            "--segid",
            *map(str, segmentation_id)
        ])
    if multiply_value is not None:
        cargs.extend([
            "--mul",
            str(multiply_value)
        ])
    if no_bb_flag:
        cargs.append("--no-bb")
    if erode_value is not None:
        cargs.extend([
            "--erode",
            str(erode_value)
        ])
    if dilate_value is not None:
        cargs.extend([
            "--dilate",
            str(dilate_value)
        ])
    if wm_flag:
        cargs.append("--wm")
    if vcsf_flag:
        cargs.append("--vcsf")
    if xcsf_flag:
        cargs.append("--xcsf")
    if remove_mean_flag:
        cargs.append("--remove-mean")
    ret = Vol2segavgOutputs(
        root=execution.output_file("."),
        output_file=execution.output_file(output_file),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VOL2SEGAVG_METADATA",
    "Vol2segavgOutputs",
    "vol2segavg",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3DMAXDISP_METADATA = Metadata(
    id="8e31f05fbdb7f51731c9b9e8013c472695fe9543.boutiques",
    name="3dmaxdisp",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dmaxdispOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3dmaxdisp(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    displacement_output: OutputPathType
    """Results showing average and maximum displacements."""


def v_3dmaxdisp(
    inset: InputPathType,
    matrix: InputPathType,
    verbose: bool = False,
    runner: Runner | None = None,
) -> V3dmaxdispOutputs:
    """
    Reads in a 3D dataset and a DICOM-based affine matrix to output the average and
    maximum displacement applied to the edge voxels of the 3D dataset's automask.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        inset: Input dataset file used to form the mask over which\
            displacements will be computed.
        matrix: 3x4 affine transformation matrix file applied to the\
            coordinates of the voxels in the dataset mask.
        verbose: Print a few progress reports.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dmaxdispOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3DMAXDISP_METADATA)
    cargs = []
    cargs.append("3dmaxdisp")
    cargs.extend([
        "-inset",
        execution.input_file(inset)
    ])
    cargs.extend([
        "-matrix",
        execution.input_file(matrix)
    ])
    if verbose:
        cargs.append("-verb")
    ret = V3dmaxdispOutputs(
        root=execution.output_file("."),
        displacement_output=execution.output_file("stdout"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dmaxdispOutputs",
    "V_3DMAXDISP_METADATA",
    "v_3dmaxdisp",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

REG_F3D_METADATA = Metadata(
    id="1e4f335840b8035582d36ee0767618d0b14f496c.boutiques",
    name="reg_f3d",
    package="niftyreg",
    container_image_tag="vnmd/niftyreg_1.4.0:20220819",
)


class RegF3dOutputs(typing.NamedTuple):
    """
    Output object returned when calling `reg_f3d(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_cpp_file: OutputPathType | None
    """File containing the output control point grid"""
    output_resampled_image_file: OutputPathType | None
    """File containing the resampled image"""


def reg_f3d(
    reference_image: InputPathType,
    floating_image: InputPathType,
    affine_transform: InputPathType | None = None,
    flirt_affine_transform: InputPathType | None = None,
    control_point_grid_input: InputPathType | None = None,
    output_cpp: str | None = None,
    output_resampled_image: str | None = None,
    reference_mask: InputPathType | None = None,
    smooth_reference: float | None = None,
    smooth_floating: float | None = None,
    num_bins_joint_histogram: float | None = None,
    num_bins_floating_joint_histogram: float | None = None,
    lower_threshold_reference: float | None = None,
    upper_threshold_reference: float | None = None,
    lower_threshold_floating: float | None = None,
    upper_threshold_floating: float | None = None,
    spacing_x: float | None = None,
    spacing_y: float | None = None,
    spacing_z: float | None = None,
    bending_energy: float | None = None,
    linear_elasticity: list[float] | None = None,
    l2_norm_displacement: float | None = None,
    jacobian_determinant: float | None = None,
    no_approx_jl: bool = False,
    no_conj: bool = False,
    ssd: bool = False,
    kld: bool = False,
    amc: bool = False,
    max_iterations: float | None = None,
    num_levels: float | None = None,
    first_levels: float | None = None,
    no_pyramid: bool = False,
    symmetric: bool = False,
    floating_mask: InputPathType | None = None,
    inverse_consistency: float | None = None,
    velocity_field: bool = False,
    composition_steps: float | None = None,
    smooth_gradient: float | None = None,
    padding_value: float | None = None,
    verbose_off: bool = False,
    runner: Runner | None = None,
) -> RegF3dOutputs:
    """
    Fast Free-Form Deformation algorithm for non-rigid registration based on
    Rueckert's 99 TMI work.
    
    Author: NiftyReg Developers
    
    URL: http://cmictig.cs.ucl.ac.uk/wiki/index.php/NiftyReg
    
    Args:
        reference_image: Filename of the reference image.
        floating_image: Filename of the floating image.
        affine_transform: Filename which contains an affine transformation.
        flirt_affine_transform: Filename which contains a flirt affine\
            transformation.
        control_point_grid_input: Filename of control point grid input.
        output_cpp: Filename of control point grid.
        output_resampled_image: Filename of the resampled image.
        reference_mask: Filename of a mask image in the reference space.
        smooth_reference: Smooth the reference image using the specified sigma\
            (mm).
        smooth_floating: Smooth the floating image using the specified sigma\
            (mm).
        num_bins_joint_histogram: Number of bins to use for the joint histogram\
            (reference).
        num_bins_floating_joint_histogram: Number of bins to use for the joint\
            histogram (floating).
        lower_threshold_reference: Lower threshold to apply to the reference\
            image intensities.
        upper_threshold_reference: Upper threshold to apply to the reference\
            image intensities.
        lower_threshold_floating: Lower threshold to apply to the floating\
            image intensities.
        upper_threshold_floating: Upper threshold to apply to the floating\
            image intensities.
        spacing_x: Final grid spacing along the x axis in mm (or in voxel if\
            negative value).
        spacing_y: Final grid spacing along the y axis in mm (or in voxel if\
            negative value).
        spacing_z: Final grid spacing along the z axis in mm (or in voxel if\
            negative value).
        bending_energy: Weight of the bending energy penalty term.
        linear_elasticity: Weights of linear elasticity penalty term.
        l2_norm_displacement: Weight of L2 norm displacement penalty term.
        jacobian_determinant: Weight of log of the Jacobian determinant penalty\
            term.
        no_approx_jl: Do not approximate the JL value only at the control point\
            position.
        no_conj: Do not use the conjugate gradient optimization but a simple\
            gradient ascent.
        ssd: Use the SSD as the similarity measure instead of NMI.
        kld: Use the KL divergence as the similarity measure instead of NMI.
        amc: Use the additive NMI for multichannel data.
        max_iterations: Maximal number of iterations per level.
        num_levels: Number of levels to perform.
        first_levels: Only perform the first levels.
        no_pyramid: Do not use a pyramidal approach.
        symmetric: Use symmetric approach.
        floating_mask: Filename of a mask image in the floating space.
        inverse_consistency: Weight of the inverse consistency penalty term.
        velocity_field: Use velocity field integration to generate the\
            deformation.
        composition_steps: Number of composition steps.
        smooth_gradient: Smooth the metric derivative (in mm).
        padding_value: Padding value.
        verbose_off: Turn verbose off.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `RegF3dOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(REG_F3D_METADATA)
    cargs = []
    cargs.append("reg_f3d")
    cargs.extend([
        "-ref",
        execution.input_file(reference_image)
    ])
    cargs.extend([
        "-flo",
        execution.input_file(floating_image)
    ])
    if affine_transform is not None:
        cargs.extend([
            "-aff",
            execution.input_file(affine_transform)
        ])
    if flirt_affine_transform is not None:
        cargs.extend([
            "-affFlirt",
            execution.input_file(flirt_affine_transform)
        ])
    if control_point_grid_input is not None:
        cargs.extend([
            "-incpp",
            execution.input_file(control_point_grid_input)
        ])
    if output_cpp is not None:
        cargs.extend([
            "-cpp",
            output_cpp
        ])
    if output_resampled_image is not None:
        cargs.extend([
            "-res",
            output_resampled_image
        ])
    if reference_mask is not None:
        cargs.extend([
            "-rmask",
            execution.input_file(reference_mask)
        ])
    if smooth_reference is not None:
        cargs.extend([
            "-smooR",
            str(smooth_reference)
        ])
    if smooth_floating is not None:
        cargs.extend([
            "-smooF",
            str(smooth_floating)
        ])
    if num_bins_joint_histogram is not None:
        cargs.extend([
            "--rbn",
            str(num_bins_joint_histogram)
        ])
    if num_bins_floating_joint_histogram is not None:
        cargs.extend([
            "--fbn",
            str(num_bins_floating_joint_histogram)
        ])
    if lower_threshold_reference is not None:
        cargs.extend([
            "--rLwTh",
            str(lower_threshold_reference)
        ])
    if upper_threshold_reference is not None:
        cargs.extend([
            "--rUpTh",
            str(upper_threshold_reference)
        ])
    if lower_threshold_floating is not None:
        cargs.extend([
            "--fLwTh",
            str(lower_threshold_floating)
        ])
    if upper_threshold_floating is not None:
        cargs.extend([
            "--fUpTh",
            str(upper_threshold_floating)
        ])
    if spacing_x is not None:
        cargs.extend([
            "-sx",
            str(spacing_x)
        ])
    if spacing_y is not None:
        cargs.extend([
            "-sy",
            str(spacing_y)
        ])
    if spacing_z is not None:
        cargs.extend([
            "-sz",
            str(spacing_z)
        ])
    if bending_energy is not None:
        cargs.extend([
            "-be",
            str(bending_energy)
        ])
    if linear_elasticity is not None:
        cargs.extend([
            "-le",
            *map(str, linear_elasticity)
        ])
    if l2_norm_displacement is not None:
        cargs.extend([
            "-l2",
            str(l2_norm_displacement)
        ])
    if jacobian_determinant is not None:
        cargs.extend([
            "-jl",
            str(jacobian_determinant)
        ])
    if no_approx_jl:
        cargs.append("-noAppJL")
    if no_conj:
        cargs.append("-noConj")
    if ssd:
        cargs.append("-ssd")
    if kld:
        cargs.append("-kld")
    if amc:
        cargs.append("-amc")
    if max_iterations is not None:
        cargs.extend([
            "-maxit",
            str(max_iterations)
        ])
    if num_levels is not None:
        cargs.extend([
            "-ln",
            str(num_levels)
        ])
    if first_levels is not None:
        cargs.extend([
            "-lp",
            str(first_levels)
        ])
    if no_pyramid:
        cargs.append("-nopy")
    if symmetric:
        cargs.append("-sym")
    if floating_mask is not None:
        cargs.extend([
            "-fmask",
            execution.input_file(floating_mask)
        ])
    if inverse_consistency is not None:
        cargs.extend([
            "-ic",
            str(inverse_consistency)
        ])
    if velocity_field:
        cargs.append("-vel")
    if composition_steps is not None:
        cargs.extend([
            "-step",
            str(composition_steps)
        ])
    if smooth_gradient is not None:
        cargs.extend([
            "-smoothGrad",
            str(smooth_gradient)
        ])
    if padding_value is not None:
        cargs.extend([
            "-pad",
            str(padding_value)
        ])
    if verbose_off:
        cargs.append("-voff")
    ret = RegF3dOutputs(
        root=execution.output_file("."),
        output_cpp_file=execution.output_file(output_cpp) if (output_cpp is not None) else None,
        output_resampled_image_file=execution.output_file(output_resampled_image) if (output_resampled_image is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "REG_F3D_METADATA",
    "RegF3dOutputs",
    "reg_f3d",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRTRANSFORM_METADATA = Metadata(
    id="faaa07c120c0bcc84911ab27778920e0d881a5ea.boutiques",
    name="mrtransform",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


@dataclasses.dataclass
class MrtransformFslgrad:
    """
    Provide the diffusion-weighted gradient scheme used in the acquisition in
    FSL bvecs/bvals format files. If a diffusion gradient scheme is present in
    the input image header, the data provided with this option will be instead
    used.
    """
    bvecs: InputPathType
    """Provide the diffusion-weighted gradient scheme used in the acquisition in
    FSL bvecs/bvals format files. If a diffusion gradient scheme is present in
    the input image header, the data provided with this option will be instead
    used."""
    bvals: InputPathType
    """Provide the diffusion-weighted gradient scheme used in the acquisition in
    FSL bvecs/bvals format files. If a diffusion gradient scheme is present in
    the input image header, the data provided with this option will be instead
    used."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-fslgrad")
        cargs.append(execution.input_file(self.bvecs))
        cargs.append(execution.input_file(self.bvals))
        return cargs


class MrtransformExportGradFslOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MrtransformExportGradFsl | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    bvecs_path: OutputPathType
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""
    bvals_path: OutputPathType
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""


@dataclasses.dataclass
class MrtransformExportGradFsl:
    """
    export the diffusion-weighted gradient table to files in FSL (bvecs / bvals)
    format.
    """
    bvecs_path: str
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""
    bvals_path: str
    """export the diffusion-weighted gradient table to files in FSL (bvecs /
    bvals) format"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-export_grad_fsl")
        cargs.append(self.bvecs_path)
        cargs.append(self.bvals_path)
        return cargs
    
    def outputs(
        self,
        execution: Execution,
    ) -> MrtransformExportGradFslOutputs:
        """
        Collect output file paths.
        
        Args:
            execution: The execution object.
        Returns:
            NamedTuple of outputs (described in `MrtransformExportGradFslOutputs`).
        """
        ret = MrtransformExportGradFslOutputs(
            root=execution.output_file("."),
            bvecs_path=execution.output_file(self.bvecs_path),
            bvals_path=execution.output_file(self.bvals_path),
        )
        return ret


@dataclasses.dataclass
class MrtransformVariousString:
    obj: str
    """String object."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append(self.obj)
        return cargs


@dataclasses.dataclass
class MrtransformVariousFile:
    obj: InputPathType
    """File object."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append(execution.input_file(self.obj))
        return cargs


@dataclasses.dataclass
class MrtransformConfig:
    """
    temporarily set the value of an MRtrix config file entry.
    """
    key: str
    """temporarily set the value of an MRtrix config file entry."""
    value: str
    """temporarily set the value of an MRtrix config file entry."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-config")
        cargs.append(self.key)
        cargs.append(self.value)
        return cargs


class MrtransformOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mrtransform(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output: OutputPathType
    """the output image."""
    export_grad_mrtrix: OutputPathType | None
    """export the diffusion-weighted gradient table to file in MRtrix format """
    export_grad_fsl: MrtransformExportGradFslOutputs | None
    """Outputs from `MrtransformExportGradFsl`."""


def mrtransform(
    input_: InputPathType,
    output: str,
    linear: InputPathType | None = None,
    flip: list[int] | None = None,
    inverse: bool = False,
    half: bool = False,
    replace: InputPathType | None = None,
    identity: bool = False,
    template: InputPathType | None = None,
    midway_space: bool = False,
    interp: str | None = None,
    oversample: list[int] | None = None,
    warp: InputPathType | None = None,
    warp_full: InputPathType | None = None,
    from_: int | None = None,
    modulate: str | None = None,
    directions: InputPathType | None = None,
    reorient_fod: str | None = None,
    grad: InputPathType | None = None,
    fslgrad: MrtransformFslgrad | None = None,
    export_grad_mrtrix: str | None = None,
    export_grad_fsl: MrtransformExportGradFsl | None = None,
    datatype: str | None = None,
    strides: typing.Union[MrtransformVariousString, MrtransformVariousFile] | None = None,
    nan: bool = False,
    no_reorientation: bool = False,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: int | None = None,
    config: list[MrtransformConfig] | None = None,
    help_: bool = False,
    version: bool = False,
    runner: Runner | None = None,
) -> MrtransformOutputs:
    """
    Apply spatial transformations to an image.
    
    If a linear transform is applied without a template image the command will
    modify the image header transform matrix
    
    FOD reorientation (with apodised point spread functions) can be performed if
    the number of volumes in the 4th dimension equals the number of coefficients
    in an antipodally symmetric spherical harmonic series (e.g. 6, 15, 28 etc).
    For such data, the -reorient_fod yes/no option must be used to specify if
    reorientation is required.
    
    The output image intensity can be modulated using the (local or global)
    volume change if a linear or nonlinear transformation is applied. 'FOD'
    modulation preserves the apparent fibre density across the fibre bundle
    width and can only be applied if FOD reorientation is used. Alternatively,
    non-directional scaling by the Jacobian determinant can be applied to any
    image type.
    
    If a DW scheme is contained in the header (or specified separately), and the
    number of directions matches the number of volumes in the images, any
    transformation applied using the -linear option will also be applied to the
    directions.
    
    When the -template option is used to specify the target image grid, the
    image provided via this option will not influence the axis data strides of
    the output image; these are determined based on the input image, or the
    input to the -strides option.
    
    References:
    
    * If FOD reorientation is being performed:
    Raffelt, D.; Tournier, J.-D.; Crozier, S.; Connelly, A. & Salvado, O.
    Reorientation of fiber orientation distributions using apodized point spread
    functions. Magnetic Resonance in Medicine, 2012, 67, 844-855
    
    * If FOD modulation is being performed:
    Raffelt, D.; Tournier, J.-D.; Rose, S.; Ridgway, G.R.; Henderson, R.;
    Crozier, S.; Salvado, O.; Connelly, A.; Apparent Fibre Density: a novel
    measure for the analysis of diffusion-weighted magnetic resonance images.
    NeuroImage, 2012, 15;59(4), 3976-94.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        input_: input image to be transformed.
        output: the output image.
        linear: specify a linear transform to apply, in the form of a 3x4 or\
            4x4 ascii file. Note the standard 'reverse' convention is used, where\
            the transform maps points in the template image to the moving image.\
            Note that the reverse convention is still assumed even if no -template\
            image is supplied.
        flip: flip the specified axes, provided as a comma-separated list of\
            indices (0:x, 1:y, 2:z).
        inverse: apply the inverse transformation.
        half: apply the matrix square root of the transformation. This can be\
            combined with the inverse option.
        replace: replace the linear transform of the original image by that\
            specified, rather than applying it to the original image. The specified\
            transform can be either a template image, or a 3x4 or 4x4 ascii file.
        identity: set the header transform of the image to the identity matrix.
        template: reslice the input image to match the specified template image\
            grid.
        midway_space: reslice the input image to the midway space. Requires\
            either the -template or -warp option. If used with -template and\
            -linear option the input image will be resliced onto the grid halfway\
            between the input and template. If used with the -warp option the input\
            will be warped to the midway space defined by the grid of the input\
            warp (i.e. half way between image1 and image2).
        interp: set the interpolation method to use when reslicing (choices:\
            nearest, linear, cubic, sinc. Default: cubic).
        oversample: set the amount of over-sampling (in the target space) to\
            perform when regridding. This is particularly relevant when downsamping\
            a high-resolution image to a low-resolution image, to avoid aliasing\
            artefacts. This can consist of a single integer, or a comma-separated\
            list of 3 integers if different oversampling factors are desired along\
            the different axes. Default is determined from ratio of voxel\
            dimensions (disabled for nearest-neighbour interpolation).
        warp: apply a non-linear 4D deformation field to warp the input image.\
            Each voxel in the deformation field must define the scanner space\
            position that will be used to interpolate the input image during\
            warping (i.e. pull-back/reverse warp convention). If the -template\
            image is also supplied the deformation field will be resliced first to\
            the template image grid. If no -template option is supplied then the\
            output image will have the same image grid as the deformation field.\
            This option can be used in combination with the -affine option, in\
            which case the affine will be applied first).
        warp_full: warp the input image using a 5D warp file output from\
            mrregister. Any linear transforms in the warp image header will also be\
            applied. The -warp_full option must be used in combination with either\
            the -template option or the -midway_space option. If a -template image\
            is supplied then the full warp will be used. By default the\
            image1->image2 transform will be applied, however the -from 2 option\
            can be used to apply the image2->image1 transform. Use the\
            -midway_space option to warp the input image to the midway space. The\
            -from option can also be used to define which warp to use when\
            transforming to midway space.
        from_: used to define which space the input image is when using the\
            -warp_mid option. Use -from 1 to warp from image1 or -from 2 to warp\
            from image2.
        modulate: Valid choices are: fod and jac.\
            fod: modulate FODs during reorientation to preserve the apparent\
            fibre density across fibre bundle widths before and after the\
            transformation.\
            jac: modulate the image intensity with the determinant of the\
            Jacobian of the warp of linear transformation to preserve the total\
            intensity before and after the transformation.
        directions: directions defining the number and orientation of the\
            apodised point spread functions used in FOD reorientation (Default: 300\
            directions).
        reorient_fod: specify whether to perform FOD reorientation. This is\
            required if the number of volumes in the 4th dimension corresponds to\
            the number of coefficients in an antipodally symmetric spherical\
            harmonic series with lmax >= 2 (i.e. 6, 15, 28, 45, 66 volumes).
        grad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in a text file. This should be supplied as a 4xN text file\
            with each line is in the format [ X Y Z b ], where [ X Y Z ] describe\
            the direction of the applied gradient, and b gives the b-value in units\
            of s/mm^2. If a diffusion gradient scheme is present in the input image\
            header, the data provided with this option will be instead used.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        export_grad_mrtrix: export the diffusion-weighted gradient table to\
            file in MRtrix format.
        export_grad_fsl: export the diffusion-weighted gradient table to files\
            in FSL (bvecs / bvals) format.
        datatype: specify output image data type. Valid choices are: float32,\
            float32le, float32be, float64, float64le, float64be, int64, uint64,\
            int64le, uint64le, int64be, uint64be, int32, uint32, int32le, uint32le,\
            int32be, uint32be, int16, uint16, int16le, uint16le, int16be, uint16be,\
            cfloat32, cfloat32le, cfloat32be, cfloat64, cfloat64le, cfloat64be,\
            int8, uint8, bit.
        strides: specify the strides of the output data in memory; either as a\
            comma-separated list of (signed) integers, or as a template image from\
            which the strides shall be extracted and used. The actual strides\
            produced will depend on whether the output image format can support it.
        nan: Use NaN as the out of bounds value (Default: 0.0).
        no_reorientation: deprecated, use -reorient_fod instead.
        info: display information messages.
        quiet: do not display information messages or progress status;\
            alternatively, this can be achieved by setting the MRTRIX_QUIET\
            environment variable to a non-empty string.
        debug: display debugging messages.
        force: force overwrite of output files (caution: using the same file as\
            input and output might cause unexpected behaviour).
        nthreads: use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: temporarily set the value of an MRtrix config file entry.
        help_: display this information page and exit.
        version: display version information and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrtransformOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRTRANSFORM_METADATA)
    cargs = []
    cargs.append("mrtransform")
    if linear is not None:
        cargs.extend([
            "-linear",
            execution.input_file(linear)
        ])
    if flip is not None:
        cargs.extend([
            "-flip",
            ",".join(map(str, flip))
        ])
    if inverse:
        cargs.append("-inverse")
    if half:
        cargs.append("-half")
    if replace is not None:
        cargs.extend([
            "-replace",
            execution.input_file(replace)
        ])
    if identity:
        cargs.append("-identity")
    if template is not None:
        cargs.extend([
            "-template",
            execution.input_file(template)
        ])
    if midway_space:
        cargs.append("-midway_space")
    if interp is not None:
        cargs.extend([
            "-interp",
            interp
        ])
    if oversample is not None:
        cargs.extend([
            "-oversample",
            ",".join(map(str, oversample))
        ])
    if warp is not None:
        cargs.extend([
            "-warp",
            execution.input_file(warp)
        ])
    if warp_full is not None:
        cargs.extend([
            "-warp_full",
            execution.input_file(warp_full)
        ])
    if from_ is not None:
        cargs.extend([
            "-from",
            str(from_)
        ])
    if modulate is not None:
        cargs.extend([
            "-modulate",
            modulate
        ])
    if directions is not None:
        cargs.extend([
            "-directions",
            execution.input_file(directions)
        ])
    if reorient_fod is not None:
        cargs.extend([
            "-reorient_fod",
            reorient_fod
        ])
    if grad is not None:
        cargs.extend([
            "-grad",
            execution.input_file(grad)
        ])
    if fslgrad is not None:
        cargs.extend(fslgrad.run(execution))
    if export_grad_mrtrix is not None:
        cargs.extend([
            "-export_grad_mrtrix",
            export_grad_mrtrix
        ])
    if export_grad_fsl is not None:
        cargs.extend(export_grad_fsl.run(execution))
    if datatype is not None:
        cargs.extend([
            "-datatype",
            datatype
        ])
    if strides is not None:
        cargs.extend([
            "-strides",
            *strides.run(execution)
        ])
    if nan:
        cargs.append("-nan")
    if no_reorientation:
        cargs.append("-no_reorientation")
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([a for c in [s.run(execution) for s in config] for a in c])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    cargs.append(execution.input_file(input_))
    cargs.append(output)
    ret = MrtransformOutputs(
        root=execution.output_file("."),
        output=execution.output_file(output),
        export_grad_mrtrix=execution.output_file(export_grad_mrtrix) if (export_grad_mrtrix is not None) else None,
        export_grad_fsl=export_grad_fsl.outputs(execution) if export_grad_fsl else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRTRANSFORM_METADATA",
    "MrtransformConfig",
    "MrtransformExportGradFsl",
    "MrtransformExportGradFslOutputs",
    "MrtransformFslgrad",
    "MrtransformOutputs",
    "MrtransformVariousFile",
    "MrtransformVariousString",
    "mrtransform",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SUSAN_METADATA = Metadata(
    id="a4d2f0dce1104c4f1c639410a4021ae8fd4eea82.boutiques",
    name="susan",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class SusanOutputs(typing.NamedTuple):
    """
    Output object returned when calling `susan(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    filtered_output: OutputPathType
    """Filtered output image file"""


def susan(
    input_file: InputPathType,
    brightness_threshold: float,
    spatial_size: float,
    dimensionality: float,
    use_median_filter: float,
    n_usans: float,
    output_file: str,
    usan1: InputPathType | None = None,
    brightness_threshold1: float | None = None,
    usan2: InputPathType | None = None,
    brightness_threshold2: float | None = None,
    runner: Runner | None = None,
) -> SusanOutputs:
    """
    Non-linear noise reduction filtering tool.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        input_file: Input image file.
        brightness_threshold: Brightness threshold; should be greater than\
            noise level and less than contrast of edges to be preserved.
        spatial_size: Spatial size (sigma, i.e., half-width) of smoothing, in\
            mm.
        dimensionality: Dimensionality (2 or 3), for within-plane (2) or fully\
            3D (3) smoothing.
        use_median_filter: Use median filter for cases where single-point noise\
            is detected (0 or 1).
        n_usans: Determine if the smoothing area is found from secondary images\
            (0, 1 or 2).
        output_file: Output image file.
        usan1: First USAN image file.
        brightness_threshold1: Brightness threshold for first USAN image.
        usan2: Second USAN image file.
        brightness_threshold2: Brightness threshold for second USAN image.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SusanOutputs`).
    """
    if not (2 <= dimensionality <= 3): 
        raise ValueError(f"'dimensionality' must be between 2 <= x <= 3 but was {dimensionality}")
    if not (0 <= use_median_filter <= 1): 
        raise ValueError(f"'use_median_filter' must be between 0 <= x <= 1 but was {use_median_filter}")
    if not (0 <= n_usans <= 2): 
        raise ValueError(f"'n_usans' must be between 0 <= x <= 2 but was {n_usans}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(SUSAN_METADATA)
    cargs = []
    cargs.append("susan")
    cargs.append(execution.input_file(input_file))
    cargs.append(str(brightness_threshold))
    cargs.append(str(spatial_size))
    cargs.append(str(dimensionality))
    cargs.append(str(use_median_filter))
    cargs.append(str(n_usans))
    if usan1 is not None:
        cargs.append(execution.input_file(usan1))
    if brightness_threshold1 is not None:
        cargs.append(str(brightness_threshold1))
    if usan2 is not None:
        cargs.append(execution.input_file(usan2))
    if brightness_threshold2 is not None:
        cargs.append(str(brightness_threshold2))
    cargs.append(output_file)
    ret = SusanOutputs(
        root=execution.output_file("."),
        filtered_output=execution.output_file(output_file),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SUSAN_METADATA",
    "SusanOutputs",
    "susan",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_RIBBON_METADATA = Metadata(
    id="63d995760cf07991e31c0b77f7e67dbab7a6b9bb.boutiques",
    name="mri_ribbon",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriRibbonOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_ribbon(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    ribbon_output: OutputPathType
    """Output ribbon volume"""


def mri_ribbon(
    inner_surface: InputPathType,
    outer_surface: InputPathType,
    input_volume: str,
    output_volume: str,
    label_file: InputPathType | None = None,
    runner: Runner | None = None,
) -> MriRibbonOutputs:
    """
    MRI Ribbon tool to create ribbon volumes from surface files.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        inner_surface: File name of the inner surface.
        outer_surface: File name of the outer surface.
        input_volume: Prefix for the input volume file.
        output_volume: Prefix for the output volume file.
        label_file: Label file specifying regions of interest (optional).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriRibbonOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_RIBBON_METADATA)
    cargs = []
    cargs.append("mri_ribbon")
    if label_file is not None:
        cargs.extend([
            "-l",
            execution.input_file(label_file)
        ])
    cargs.append(execution.input_file(inner_surface))
    cargs.append(execution.input_file(outer_surface))
    cargs.append(input_volume)
    cargs.append(output_volume)
    ret = MriRibbonOutputs(
        root=execution.output_file("."),
        ribbon_output=execution.output_file(output_volume + "_ribbon.nii.gz"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_RIBBON_METADATA",
    "MriRibbonOutputs",
    "mri_ribbon",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

UNPACKMINCDIR_METADATA = Metadata(
    id="665926cbe3f5b98854ac7511f567b75246bc682e.boutiques",
    name="unpackmincdir",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class UnpackmincdirOutputs(typing.NamedTuple):
    """
    Output object returned when calling `unpackmincdir(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def unpackmincdir(
    source_directory: str,
    target_directory: str,
    scan_sequence_info: str | None = None,
    functional_sequence: str | None = None,
    functional_subdirectory: str | None = None,
    minc_only: bool = False,
    no_copy: bool = False,
    umask: str | None = None,
    runner: Runner | None = None,
) -> UnpackmincdirOutputs:
    """
    Tool for unpacking directories with MINC files.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        source_directory: Source directory containing the MINC files.
        target_directory: Target directory to unpack the contents to.
        scan_sequence_info: Scan sequence directives file, e.g.,\
            freesurfer_alpha/scanseq.info.
        functional_sequence: Use seqname for functionals (example:\
            ep2d_fid_ts_20b2604).
        functional_subdirectory: Functional subdirectory, e.g., bold.
        minc_only: Do not unpack into bshorts.
        no_copy: Create directories but do not copy/convert data.
        umask: Unix file permission mask (default: 22).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `UnpackmincdirOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(UNPACKMINCDIR_METADATA)
    cargs = []
    cargs.append("unpackmincdir")
    cargs.extend([
        "-src",
        source_directory
    ])
    cargs.extend([
        "-targ",
        target_directory
    ])
    if scan_sequence_info is not None:
        cargs.extend([
            "-scanseqinfo",
            scan_sequence_info
        ])
    if functional_sequence is not None:
        cargs.extend([
            "-funcseq",
            functional_sequence
        ])
    if functional_subdirectory is not None:
        cargs.extend([
            "-fsd",
            functional_subdirectory
        ])
    if minc_only:
        cargs.append("-minconly")
    if no_copy:
        cargs.append("-nocopy")
    if umask is not None:
        cargs.extend([
            "-umask",
            umask
        ])
    ret = UnpackmincdirOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "UNPACKMINCDIR_METADATA",
    "UnpackmincdirOutputs",
    "unpackmincdir",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

BAYES_VIEW_METADATA = Metadata(
    id="6b7c34a2ea4931bfe8ed6b989b43904d7d18f61b.boutiques",
    name="bayes_view",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class BayesViewOutputs(typing.NamedTuple):
    """
    Output object returned when calling `bayes_view(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def bayes_view(
    input_folder: str,
    help_: bool = False,
    shiny_folder: str | None = None,
    runner: Runner | None = None,
) -> BayesViewOutputs:
    """
    Launch a shiny app to visualize RBA output files. The files must have the .RData
    extension.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        input_folder: Path to a folder containing .RData files.
        help_: Show help message.
        shiny_folder: Use a custom shiny folder (for testing purposes).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BayesViewOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(BAYES_VIEW_METADATA)
    cargs = []
    cargs.append("bayes_view")
    cargs.append(input_folder)
    if help_:
        cargs.append("-help")
    if shiny_folder is not None:
        cargs.extend([
            "-ShinyFolder",
            shiny_folder
        ])
    ret = BayesViewOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "BAYES_VIEW_METADATA",
    "BayesViewOutputs",
    "bayes_view",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FS_TEMP_FILE_METADATA = Metadata(
    id="517fdbcd5c110803a94e295435754191461fd4ee.boutiques",
    name="fs_temp_file",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class FsTempFileOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fs_temp_file(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def fs_temp_file(
    base_dir_alt: str | None = None,
    suffix_alt: str | None = None,
    scratch: bool = False,
    help_alt: bool = False,
    runner: Runner | None = None,
) -> FsTempFileOutputs:
    """
    Generates and creates an empty temporary file, printing the resulting path to
    stdout.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        base_dir_alt: Manually specify base temporary directory.
        suffix_alt: Optional file suffix.
        scratch: Use /scratch directory if available, but FS_TMPDIR takes\
            priority.
        help_alt: Print help text and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FsTempFileOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FS_TEMP_FILE_METADATA)
    cargs = []
    cargs.append("fs_temp_file")
    if base_dir_alt is not None:
        cargs.extend([
            "--base",
            base_dir_alt
        ])
    if suffix_alt is not None:
        cargs.extend([
            "--suffix",
            suffix_alt
        ])
    if scratch:
        cargs.append("--scratch")
    if help_alt:
        cargs.append("--help")
    ret = FsTempFileOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FS_TEMP_FILE_METADATA",
    "FsTempFileOutputs",
    "fs_temp_file",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

SEGMENT_BS_SH_METADATA = Metadata(
    id="ea5de6d87d6861bdf790a471d88443f6b20986d7.boutiques",
    name="segmentBS.sh",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class SegmentBsShOutputs(typing.NamedTuple):
    """
    Output object returned when calling `segment_bs_sh(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def segment_bs_sh(
    matlab_runtime: str | None = "/usr/local/freesurfer/MCRv97",
    runner: Runner | None = None,
) -> SegmentBsShOutputs:
    """
    Segmentation tool for hippocampal/amygdala and brainstem modules.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        matlab_runtime: Path to the Matlab 2019b runtime environment; necessary\
            for running the segmentation tool.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SegmentBsShOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(SEGMENT_BS_SH_METADATA)
    cargs = []
    cargs.append("segmentBS.sh")
    if matlab_runtime is not None:
        cargs.append(matlab_runtime)
    ret = SegmentBsShOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "SEGMENT_BS_SH_METADATA",
    "SegmentBsShOutputs",
    "segment_bs_sh",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

TRAC_PATHS_METADATA = Metadata(
    id="e536dea8d1b847571fb37c90fba034bc59556c4c.boutiques",
    name="trac-paths",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class TracPathsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `trac_paths(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def trac_paths(
    dmrirc_file: InputPathType,
    log_file: str | None = None,
    no_log: bool = False,
    cmd_file: str | None = None,
    no_cmd: bool = False,
    no_isrunning: bool = False,
    umask: str | None = None,
    group_id: str | None = None,
    allow_core_dump: bool = False,
    debug: bool = False,
    dontrun: bool = False,
    version: bool = False,
    help_: bool = False,
    runner: Runner | None = None,
) -> TracPathsOutputs:
    """
    Tractography for a single subject.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        dmrirc_file: dmrirc file (see dmrirc.example).
        log_file: Log file, default is trac-all.log in the same directory as\
            dmrirc.
        no_log: Do not save a log file.
        cmd_file: Cmd file, default is trac-all.cmd in the same directory as\
            dmrirc.
        no_cmd: Do not save a cmd file.
        no_isrunning: Do not check whether this subject is currently being\
            processed.
        umask: Set Unix file permission mask (default 002).
        group_id: Check that current group is alpha groupid.
        allow_core_dump: Set coredump limit to unlimited.
        debug: Generate much more output.
        dontrun: Do everything but execute each command.
        version: Print version of this script and exit.
        help_: Print full contents of help.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `TracPathsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(TRAC_PATHS_METADATA)
    cargs = []
    cargs.append("trac-paths")
    cargs.extend([
        "-c",
        execution.input_file(dmrirc_file)
    ])
    if log_file is not None:
        cargs.extend([
            "-log",
            log_file
        ])
    if no_log:
        cargs.append("-nolog")
    if cmd_file is not None:
        cargs.extend([
            "-cmd",
            cmd_file
        ])
    if no_cmd:
        cargs.append("-nocmd")
    if no_isrunning:
        cargs.append("-no-isrunning")
    if umask is not None:
        cargs.extend([
            "-umask",
            umask
        ])
    if group_id is not None:
        cargs.extend([
            "-grp",
            group_id
        ])
    if allow_core_dump:
        cargs.append("-allowcoredump")
    if debug:
        cargs.append("-debug")
    if dontrun:
        cargs.append("-dontrun")
    if version:
        cargs.append("-version")
    if help_:
        cargs.append("-help")
    ret = TracPathsOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "TRAC_PATHS_METADATA",
    "TracPathsOutputs",
    "trac_paths",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

CIFTI_MERGE_METADATA = Metadata(
    id="9084c8a8cff6894fb50075bd51ea6925c1f6b4d6.boutiques",
    name="cifti-merge",
    package="workbench",
    container_image_tag="brainlife/connectome_workbench:1.5.0-freesurfer-update",
)


@dataclasses.dataclass
class CiftiMergeUpTo:
    """
    use an inclusive range of indices.
    """
    last_index: str
    """the number or name of the last index to include"""
    opt_reverse: bool = False
    """use the range in reverse order"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-up-to")
        cargs.append(self.last_index)
        if self.opt_reverse:
            cargs.append("-reverse")
        return cargs


@dataclasses.dataclass
class CiftiMergeIndex:
    """
    select a single index to use.
    """
    index: str
    """the index number (starting from 1), or name"""
    up_to: CiftiMergeUpTo | None = None
    """use an inclusive range of indices"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-index")
        cargs.append(self.index)
        if self.up_to is not None:
            cargs.extend(self.up_to.run(execution))
        return cargs


@dataclasses.dataclass
class CiftiMergeCifti:
    """
    specify an input cifti file.
    """
    cifti_in: InputPathType
    """a cifti file to use data from"""
    index: list[CiftiMergeIndex] | None = None
    """select a single index to use"""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-cifti")
        cargs.append(execution.input_file(self.cifti_in))
        if self.index is not None:
            cargs.extend([a for c in [s.run(execution) for s in self.index] for a in c])
        return cargs


class CiftiMergeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `cifti_merge(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """output cifti file"""


def cifti_merge(
    cifti_out: str,
    opt_direction_direction: str | None = None,
    opt_mem_limit_limit_gb: float | None = None,
    cifti: list[CiftiMergeCifti] | None = None,
    runner: Runner | None = None,
) -> CiftiMergeOutputs:
    """
    Merge or split on series, scalar, or label dimensions.
    
    Given input CIFTI files for which mappings along the selected direction are
    the same type, all either series, scalars, or labels, and the other
    dimensions are index-compatible, this command concatenates the data in the
    specified indices/ranges along the selected direction (by default, on
    typical 2D cifti, concatenate horizontally, so rows become longer). The
    direction can be either an integer starting from 1, or the strings 'ROW' or
    'COLUMN'.
    
    Example: wb_command -cifti-merge out.dtseries.nii -cifti first.dtseries.nii
    -index 1 -cifti second.dtseries.nii
    
    This example would take the first column from first.dtseries.nii, followed
    by all columns from second.dtseries.nii, and write these columns to
    out.dtseries.nii. .
    
    Author: Connectome Workbench Developers
    
    URL: https://github.com/Washington-University/workbench
    
    Args:
        cifti_out: output cifti file.
        opt_direction_direction: merge in a direction other than along rows:\
            the dimension to split/concatenate along, default ROW.
        opt_mem_limit_limit_gb: restrict memory used for file reading\
            efficiency: memory limit in gigabytes.
        cifti: specify an input cifti file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiMergeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_MERGE_METADATA)
    cargs = []
    cargs.append("wb_command")
    cargs.append("-cifti-merge")
    cargs.append(cifti_out)
    if opt_direction_direction is not None:
        cargs.extend([
            "-direction",
            opt_direction_direction
        ])
    if opt_mem_limit_limit_gb is not None:
        cargs.extend([
            "-mem-limit",
            str(opt_mem_limit_limit_gb)
        ])
    if cifti is not None:
        cargs.extend([a for c in [s.run(execution) for s in cifti] for a in c])
    ret = CiftiMergeOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(cifti_out),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "CIFTI_MERGE_METADATA",
    "CiftiMergeCifti",
    "CiftiMergeIndex",
    "CiftiMergeOutputs",
    "CiftiMergeUpTo",
    "cifti_merge",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_COMPUTE_LAYER_INTENSITIES_METADATA = Metadata(
    id="ea3678d2dae6244999c4877f8c133e28845c7c8b.boutiques",
    name="mris_compute_layer_intensities",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisComputeLayerIntensitiesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_compute_layer_intensities(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_overlay_file: OutputPathType
    """The output overlay file generated by the tool."""


def mris_compute_layer_intensities(
    input_intensity_volume: InputPathType,
    layer_volume_fractions_file: InputPathType,
    input_surface: InputPathType,
    output_overlay: str,
    runner: Runner | None = None,
) -> MrisComputeLayerIntensitiesOutputs:
    """
    Computes intensity overlays for specified cortical layers based on input volumes
    and surfaces.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_intensity_volume: The input intensity volume file.
        layer_volume_fractions_file: The layer volume fractions file.
        input_surface: The input surface file for layer intensity computation.
        output_overlay: The output file where the overlay will be saved.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisComputeLayerIntensitiesOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_COMPUTE_LAYER_INTENSITIES_METADATA)
    cargs = []
    cargs.append("mris_compute_layer_intensities")
    cargs.append(execution.input_file(input_intensity_volume))
    cargs.append(execution.input_file(layer_volume_fractions_file))
    cargs.append(execution.input_file(input_surface))
    cargs.append(output_overlay)
    ret = MrisComputeLayerIntensitiesOutputs(
        root=execution.output_file("."),
        output_overlay_file=execution.output_file(output_overlay),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_COMPUTE_LAYER_INTENSITIES_METADATA",
    "MrisComputeLayerIntensitiesOutputs",
    "mris_compute_layer_intensities",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

UNPACKIMADIR_METADATA = Metadata(
    id="6a27f64ee58627da90e1a875d70fa5b839c021e5.boutiques",
    name="unpackimadir",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class UnpackimadirOutputs(typing.NamedTuple):
    """
    Output object returned when calling `unpackimadir(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def unpackimadir(
    source_directory: str,
    target_directory: str,
    runner: Runner | None = None,
) -> UnpackimadirOutputs:
    """
    Unpack image directories.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        source_directory: Source directory containing the images to be unpacked.
        target_directory: Target directory where the unpacked images will be\
            stored.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `UnpackimadirOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(UNPACKIMADIR_METADATA)
    cargs = []
    cargs.append("unpackimadir")
    cargs.extend([
        "-src",
        source_directory
    ])
    cargs.extend([
        "-targ",
        target_directory
    ])
    ret = UnpackimadirOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "UNPACKIMADIR_METADATA",
    "UnpackimadirOutputs",
    "unpackimadir",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_SEGREG_METADATA = Metadata(
    id="f2669e04710c9f1f9d1fd46259ec2d013c37daf3.boutiques",
    name="mri_segreg",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriSegregOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_segreg(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    outfile: OutputPathType
    """Output file generated by mri_segreg"""


def mri_segreg(
    input_file: InputPathType,
    output_file: str = "output.mgz",
    runner: Runner | None = None,
) -> MriSegregOutputs:
    """
    MRI Segregation tool.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        input_file: Input image file. All formats accepted by mri_convert can\
            be used.
        output_file: Output file for the processed image.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriSegregOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_SEGREG_METADATA)
    cargs = []
    cargs.append("mri_segreg")
    cargs.append(execution.input_file(input_file))
    cargs.append(output_file)
    ret = MriSegregOutputs(
        root=execution.output_file("."),
        outfile=execution.output_file(output_file),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_SEGREG_METADATA",
    "MriSegregOutputs",
    "mri_segreg",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V_3D_AUTO_TCORRELATE_METADATA = Metadata(
    id="54f323538871e62136cb28afd7fdc9c55ace001b.boutiques",
    name="3dAutoTcorrelate",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class V3dAutoTcorrelateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v_3d_auto_tcorrelate(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_brick: OutputPathType | None
    """Main output dataset"""
    output_head: OutputPathType | None
    """Header information for main output dataset"""
    out1d_file: OutputPathType | None
    """Output in 1D text format"""


def v_3d_auto_tcorrelate(
    input_dataset: InputPathType,
    pearson: bool = False,
    eta2: bool = False,
    polort: int | None = None,
    autoclip: bool = False,
    automask: bool = False,
    mask: InputPathType | None = None,
    mask_only_targets: bool = False,
    mask_source: InputPathType | None = None,
    prefix: str | None = None,
    out1d: str | None = None,
    time_: bool = False,
    mmap_: bool = False,
    runner: Runner | None = None,
) -> V3dAutoTcorrelateOutputs:
    """
    Computes the correlation coefficient between the time series of each pair of
    voxels in the input dataset.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        input_dataset: Input dataset.
        pearson: Correlation is the normal Pearson (product moment) correlation\
            coefficient [default].
        eta2: Output is eta^2 measure from Cohen et al., NeuroImage, 2008.
        polort: Remove polynomial trend of order 'm', for m=-1..3.
        autoclip: Clip off low-intensity regions in the dataset.
        automask: Apply automask to the dataset.
        mask: Mask of both 'source' and 'target' voxels.
        mask_only_targets: Provide output for all voxels.
        mask_source: Provide output for voxels only in specified mask.
        prefix: Save output into dataset with specified prefix.
        out1d: Save output in a text file in 1D format.
        time_: Mark output as a 3D+time dataset.
        mmap_: Write .BRIK results to disk directly using Unix mmap().
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `V3dAutoTcorrelateOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V_3D_AUTO_TCORRELATE_METADATA)
    cargs = []
    cargs.append("3dAutoTcorrelate")
    cargs.append(execution.input_file(input_dataset))
    if pearson:
        cargs.append("-pearson")
    if eta2:
        cargs.append("-eta2")
    if polort is not None:
        cargs.extend([
            "-polort",
            str(polort)
        ])
    if autoclip:
        cargs.append("-autoclip")
    if automask:
        cargs.append("-automask")
    if mask is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask)
        ])
    if mask_only_targets:
        cargs.append("-mask_only_targets")
    if mask_source is not None:
        cargs.extend([
            "-mask_source",
            execution.input_file(mask_source)
        ])
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    if out1d is not None:
        cargs.extend([
            "-out1D",
            out1d
        ])
    if time_:
        cargs.append("-time")
    if mmap_:
        cargs.append("-mmap")
    ret = V3dAutoTcorrelateOutputs(
        root=execution.output_file("."),
        output_brick=execution.output_file(prefix + ".BRIK") if (prefix is not None) else None,
        output_head=execution.output_file(prefix + ".HEAD") if (prefix is not None) else None,
        out1d_file=execution.output_file(out1d) if (out1d is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "V3dAutoTcorrelateOutputs",
    "V_3D_AUTO_TCORRELATE_METADATA",
    "v_3d_auto_tcorrelate",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

GEMS_COMPUTE_ATLAS_PROBS_METADATA = Metadata(
    id="e4708282a3cd6c2187910866f5c82a92aef1d963.boutiques",
    name="gems_compute_atlas_probs",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class GemsComputeAtlasProbsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `gems_compute_atlas_probs(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def gems_compute_atlas_probs(
    subjects_dir: str,
    mesh_collections: list[str],
    out_dir: str,
    segmentations_dir: str | None = None,
    gt_from_fs: bool = False,
    segmentation_name: str | None = None,
    multi_structure: bool = False,
    labels: list[str] | None = None,
    from_samseg: bool = False,
    em_iterations: float | None = None,
    show_figs: bool = False,
    save_figs: bool = False,
    save_average_figs: bool = False,
    subjects_file: str | None = None,
    labels_file: str | None = None,
    samseg_subdir: str | None = None,
    runner: Runner | None = None,
) -> GemsComputeAtlasProbsOutputs:
    """
    Tool to compute atlas probabilities using SAMSEG results.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subjects_dir: Directory with saved SAMSEG runs with --history flag.
        mesh_collections: Mesh collection file(s).
        out_dir: Output directory.
        segmentations_dir: Directory with GT segmentations.
        gt_from_fs: GT from FreeSurfer segmentations.
        segmentation_name: Filename of the segmentations, assumed to be the\
            same for each subject.
        multi_structure: Estimate alphas from more than 1 structure.
        labels: Label numbers for multi-structure estimation.
        from_samseg: SAMSEG runs obtained from command samseg instead of\
            run_samseg.
        em_iterations: Number of EM iterations.
        show_figs: Show figures during the run.
        save_figs: Save rasterized prior of each subject.
        save_average_figs: Save average rasterized prior.
        subjects_file: Text file with list of subjects.
        labels_file: Text file with list of labels (instead of --labels).
        samseg_subdir: Name of samseg subdir in subject/mri folder.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `GemsComputeAtlasProbsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(GEMS_COMPUTE_ATLAS_PROBS_METADATA)
    cargs = []
    cargs.append("gems_compute_atlas_probs")
    cargs.extend([
        "--subjects-dir",
        subjects_dir
    ])
    cargs.extend([
        "--mesh-collections",
        *mesh_collections
    ])
    cargs.extend([
        "--out-dir",
        out_dir
    ])
    if segmentations_dir is not None:
        cargs.extend([
            "--segmentations-dir",
            segmentations_dir
        ])
    if gt_from_fs:
        cargs.append("--gt-from-FS")
    if segmentation_name is not None:
        cargs.extend([
            "--segmentation-name",
            segmentation_name
        ])
    if multi_structure:
        cargs.append("--multi-structure")
    if labels is not None:
        cargs.extend([
            "--labels",
            *labels
        ])
    if from_samseg:
        cargs.append("--from-samseg")
    if em_iterations is not None:
        cargs.extend([
            "--EM-iterations",
            str(em_iterations)
        ])
    if show_figs:
        cargs.append("--show-figs")
    if save_figs:
        cargs.append("--save-figs")
    if save_average_figs:
        cargs.append("--save-average-figs")
    if subjects_file is not None:
        cargs.extend([
            "--subjects_file",
            subjects_file
        ])
    if labels_file is not None:
        cargs.extend([
            "--labels_file",
            labels_file
        ])
    if samseg_subdir is not None:
        cargs.extend([
            "--samseg-subdir",
            samseg_subdir
        ])
    ret = GemsComputeAtlasProbsOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "GEMS_COMPUTE_ATLAS_PROBS_METADATA",
    "GemsComputeAtlasProbsOutputs",
    "gems_compute_atlas_probs",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

DWIBIASCORRECT_METADATA = Metadata(
    id="e83d9f4b1c64834e7ce4b8084fa8aefaad7cc322.boutiques",
    name="dwibiascorrect",
    package="mrtrix",
    container_image_tag="mrtrix3/mrtrix3:3.0.4",
)


@dataclasses.dataclass
class DwibiascorrectFslgrad:
    """
    Provide the diffusion-weighted gradient scheme used in the acquisition in
    FSL bvecs/bvals format files. If a diffusion gradient scheme is present in
    the input image header, the data provided with this option will be instead
    used.
    """
    bvecs: InputPathType
    """Provide the diffusion-weighted gradient scheme used in the acquisition in
    FSL bvecs/bvals format files. If a diffusion gradient scheme is present in
    the input image header, the data provided with this option will be instead
    used."""
    bvals: InputPathType
    """Provide the diffusion-weighted gradient scheme used in the acquisition in
    FSL bvecs/bvals format files. If a diffusion gradient scheme is present in
    the input image header, the data provided with this option will be instead
    used."""
    
    def run(
        self,
        execution: Execution,
    ) -> list[str]:
        """
        Build command line arguments. This method is called by the main command.
        
        Args:
            execution: The execution object.
        Returns:
            Command line arguments
        """
        cargs = []
        cargs.append("-fslgrad")
        cargs.append(execution.input_file(self.bvecs))
        cargs.append(execution.input_file(self.bvals))
        return cargs


class DwibiascorrectOutputs(typing.NamedTuple):
    """
    Output object returned when calling `dwibiascorrect(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_image_file: OutputPathType
    """File containing the corrected image series"""
    bias_image_file: OutputPathType | None
    """File containing the estimated bias field"""


def dwibiascorrect(
    algorithm: str,
    input_image: InputPathType,
    output_image: str,
    grad: InputPathType | None = None,
    fslgrad: DwibiascorrectFslgrad | None = None,
    mask_image: InputPathType | None = None,
    bias_image: InputPathType | None = None,
    nocleanup: bool = False,
    scratch_dir: InputPathType | None = None,
    continue_scratch_dir: list[InputPathType] | None = None,
    info: bool = False,
    quiet: bool = False,
    debug: bool = False,
    force: bool = False,
    nthreads: float | None = None,
    config: list[str] | None = None,
    help_: bool = False,
    version: bool = False,
    ants_b: str | None = None,
    ants_c: str | None = None,
    ants_s: str | None = None,
    runner: Runner | None = None,
) -> DwibiascorrectOutputs:
    """
    Perform B1 field inhomogeneity correction for a DWI volume series using either
    ANTs or FSL.
    
    Author: MRTrix3 Developers
    
    URL: https://www.mrtrix.org/
    
    Args:
        algorithm: Select the algorithm to be used for bias correction. Options\
            are: ants, fsl.
        input_image: The input image series to be corrected.
        output_image: The output corrected image series.
        grad: Provide the diffusion gradient table in MRtrix format.
        fslgrad: Provide the diffusion-weighted gradient scheme used in the\
            acquisition in FSL bvecs/bvals format files. If a diffusion gradient\
            scheme is present in the input image header, the data provided with\
            this option will be instead used.
        mask_image: Manually provide a mask image for bias field estimation.
        bias_image: Output the estimated bias field.
        nocleanup: Do not delete intermediate files during script execution,\
            and do not delete scratch directory at script completion.
        scratch_dir: Manually specify the path in which to generate the scratch\
            directory.
        continue_scratch_dir: Continue the script from a previous execution;\
            must provide the scratch directory path.
        info: Display information messages.
        quiet: Do not display information messages or progress status.
        debug: Display debugging messages.
        force: Force overwrite of output files.
        nthreads: Use this number of threads in multi-threaded applications\
            (set to 0 to disable multi-threading).
        config: Temporarily set the value of an MRtrix config file entry.
        help_: Display help information and exit.
        version: Display version information and exit.
        ants_b: N4BiasFieldCorrection option -b (initial mesh resolution in mm,\
            spline order).
        ants_c: N4BiasFieldCorrection option -c (number of iterations,\
            convergence threshold).
        ants_s: N4BiasFieldCorrection option -s (shrink-factor applied to\
            spatial dimensions).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `DwibiascorrectOutputs`).
    """
    if continue_scratch_dir is not None and (len(continue_scratch_dir) != 2): 
        raise ValueError(f"Length of 'continue_scratch_dir' must be 2 but was {len(continue_scratch_dir)}")
    runner = runner or get_global_runner()
    execution = runner.start_execution(DWIBIASCORRECT_METADATA)
    cargs = []
    cargs.append("dwibiascorrect")
    cargs.append(algorithm)
    cargs.append(execution.input_file(input_image))
    cargs.append(output_image)
    if grad is not None:
        cargs.extend([
            "-grad",
            execution.input_file(grad)
        ])
    if fslgrad is not None:
        cargs.extend(fslgrad.run(execution))
    if mask_image is not None:
        cargs.extend([
            "-mask",
            execution.input_file(mask_image)
        ])
    if bias_image is not None:
        cargs.extend([
            "-bias",
            execution.input_file(bias_image)
        ])
    if nocleanup:
        cargs.append("-nocleanup")
    if scratch_dir is not None:
        cargs.extend([
            "-scratch",
            execution.input_file(scratch_dir)
        ])
    if continue_scratch_dir is not None:
        cargs.extend([
            "-continue",
            *[execution.input_file(f) for f in continue_scratch_dir]
        ])
    if info:
        cargs.append("-info")
    if quiet:
        cargs.append("-quiet")
    if debug:
        cargs.append("-debug")
    if force:
        cargs.append("-force")
    if nthreads is not None:
        cargs.extend([
            "-nthreads",
            str(nthreads)
        ])
    if config is not None:
        cargs.extend([
            "-config",
            *config
        ])
    if help_:
        cargs.append("-help")
    if version:
        cargs.append("-version")
    if ants_b is not None:
        cargs.extend([
            "-ants.b",
            ants_b
        ])
    if ants_c is not None:
        cargs.extend([
            "-ants.c",
            ants_c
        ])
    if ants_s is not None:
        cargs.extend([
            "-ants.s",
            ants_s
        ])
    ret = DwibiascorrectOutputs(
        root=execution.output_file("."),
        output_image_file=execution.output_file(output_image),
        bias_image_file=execution.output_file(pathlib.Path(bias_image).name) if (bias_image is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "DWIBIASCORRECT_METADATA",
    "DwibiascorrectFslgrad",
    "DwibiascorrectOutputs",
    "dwibiascorrect",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

BAYESIAN_GROUP_ANA_PY_METADATA = Metadata(
    id="e49086bdbe64c248fa7b140fd8ae072e880fa9a7.boutiques",
    name="BayesianGroupAna.py",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class BayesianGroupAnaPyOutputs(typing.NamedTuple):
    """
    Output object returned when calling `bayesian_group_ana_py(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    summary: OutputPathType | None
    """Summary of the brmsfit object from R."""
    rhats: OutputPathType | None
    """Rhats for each effect and x variable combination."""
    intercept_table: OutputPathType | None
    """Table with the MedianEst, StdDev, 2.50%, 5%, 50%, 95%, and 97.50% of each
    ROI for the Intercept term."""
    x_var_table: OutputPathType | None
    """The same table as the Intercept but for the specified x variable."""


def bayesian_group_ana_py(
    data_table: InputPathType,
    y_variable: str,
    prefix: str | None = None,
    x_variables: list[str] | None = None,
    no_center: bool = False,
    iterations: float | None = None,
    chains: float | None = None,
    control_list: str | None = None,
    plot: bool = False,
    more_plots: list[str] | None = None,
    rdata: bool = False,
    seed: float | None = None,
    overwrite: bool = False,
    help_: bool = False,
    runner: Runner | None = None,
) -> BayesianGroupAnaPyOutputs:
    """
    This program conducts Bayesian Group Analysis (BGA) on a list of regions of
    interest (ROIs). Compared to the conventional univariate GLM, BGA pools and
    shares the information across the ROIs in a multilevel system.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        data_table: Input text file containing the data table.
        y_variable: Column name for the y variable.
        prefix: Name of the output file.
        x_variables: Column name(s) for the x variables. If not specified, only\
            the intercept will be added.
        no_center: Disable centering on the x variables. Maybe useful if you\
            centered manually.
        iterations: Number of total iterations per chain including warmup.\
            Default [1000].
        chains: Number of Markov chains. Default [4].
        control_list: Comma separated list of control parameters to pass to the\
            brm function. Default is the brm function defaults.
        plot: Output box, fit, and posterior prediction plots.
        more_plots: Output 'stanplots' given different types of plot names.
        rdata: Save the R session workspace and data.
        seed: Seed to generate random number. Default [1234].
        overwrite: Overwrites the output files.
        help_: Show help message and exit.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `BayesianGroupAnaPyOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(BAYESIAN_GROUP_ANA_PY_METADATA)
    cargs = []
    cargs.append("BayesianGroupAna.py")
    cargs.append(execution.input_file(data_table))
    cargs.append(y_variable)
    if prefix is not None:
        cargs.extend([
            "-prefix",
            prefix
        ])
    if x_variables is not None:
        cargs.extend([
            "-x",
            *x_variables
        ])
    if no_center:
        cargs.append("-no_center")
    if iterations is not None:
        cargs.extend([
            "-iterations",
            str(iterations)
        ])
    if chains is not None:
        cargs.extend([
            "-chains",
            str(chains)
        ])
    if control_list is not None:
        cargs.extend([
            "-control_list",
            control_list
        ])
    if plot:
        cargs.append("-plot")
    if more_plots is not None:
        cargs.extend([
            "-more_plots",
            *more_plots
        ])
    if rdata:
        cargs.append("-RData")
    if seed is not None:
        cargs.extend([
            "-seed",
            str(seed)
        ])
    if overwrite:
        cargs.append("-overwrite")
    if help_:
        cargs.append("-help")
    ret = BayesianGroupAnaPyOutputs(
        root=execution.output_file("."),
        summary=execution.output_file(prefix + "_summary.txt") if (prefix is not None) else None,
        rhats=execution.output_file(prefix + "_rhats.csv") if (prefix is not None) else None,
        intercept_table=execution.output_file(prefix + "_Intercept_table.csv") if (prefix is not None) else None,
        x_var_table=execution.output_file(prefix + "_table.csv") if (prefix is not None) else None,
    )
    execution.run(cargs)
    return ret


__all__ = [
    "BAYESIAN_GROUP_ANA_PY_METADATA",
    "BayesianGroupAnaPyOutputs",
    "bayesian_group_ana_py",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRI_LABEL_VOLUME_METADATA = Metadata(
    id="c74d81eab6ea4f91d999a3f527f7e857de7b9442.boutiques",
    name="mri_label_volume",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MriLabelVolumeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mri_label_volume(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_volume_results: OutputPathType
    """Computed volume results."""
    output_log_results: OutputPathType
    """Log file if -l option is used."""


def mri_label_volume(
    volume: InputPathType,
    labels: list[str],
    partial_volume_effects: InputPathType | None = None,
    intracranial_volume: InputPathType | None = None,
    spreadsheet_subject: str | None = None,
    non_zero_voxels: bool = False,
    replace_label_in: str | None = None,
    replace_label_out: str | None = None,
    brain_volume: InputPathType | None = None,
    percentage: bool = False,
    log_results: InputPathType | None = None,
    atlas_transform_file: InputPathType | None = None,
    atlas_scalefactor: float | None = None,
    etiv_transform_file: InputPathType | None = None,
    etiv_scalefactor: float | None = None,
    etiv_subject: str | None = None,
    runner: Runner | None = None,
) -> MriLabelVolumeOutputs:
    """
    A tool to compute volumes of labeled voxels within MRI images, often used in
    conjunction with FreeSurfer processed data.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        volume: Volume file (e.g., volume.mgz) to analyze.
        labels: Labels to compute volume for.
        partial_volume_effects: Compute partial volume effects using intensity\
            volume.
        intracranial_volume: Normalize by the intracranial volume in provided\
            file.
        spreadsheet_subject: Output in spreadsheet mode, including subject name.
        non_zero_voxels: Compute volume of all non-zero voxels (e.g. for\
            computing brain volume).
        replace_label_in: Replace label <in> with label <out>.
        replace_label_out: Label to replace with.
        brain_volume: Compute the brain volume from provided brain volume file\
            and normalize by it.
        percentage: Compute volume as a percentage of all non-zero labels.
        log_results: Log results to provided file.
        atlas_transform_file: Specify LTA or XFM atlas transform file and scale\
            factor to use for ICV correction.
        atlas_scalefactor: Scale factor for ICV correction.
        etiv_transform_file: Same as -atlas_icv.
        etiv_scalefactor: eTIV scale factor.
        etiv_subject: Same as -eTIV, and generate MATLAB data appending subject\
            to structure.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MriLabelVolumeOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRI_LABEL_VOLUME_METADATA)
    cargs = []
    cargs.append("mri_label_volume")
    cargs.append(execution.input_file(volume))
    cargs.extend(labels)
    if partial_volume_effects is not None:
        cargs.extend([
            "-pv",
            execution.input_file(partial_volume_effects)
        ])
    if intracranial_volume is not None:
        cargs.extend([
            "-icv",
            execution.input_file(intracranial_volume)
        ])
    if spreadsheet_subject is not None:
        cargs.extend([
            "-s",
            spreadsheet_subject
        ])
    if non_zero_voxels:
        cargs.append("-a")
    if replace_label_in is not None:
        cargs.extend([
            "-t",
            replace_label_in
        ])
    if replace_label_out is not None:
        cargs.append(replace_label_out)
    if brain_volume is not None:
        cargs.extend([
            "-b",
            execution.input_file(brain_volume)
        ])
    if percentage:
        cargs.append("-p")
    if log_results is not None:
        cargs.extend([
            "-l",
            execution.input_file(log_results)
        ])
    if atlas_transform_file is not None:
        cargs.extend([
            "-atlas_icv",
            execution.input_file(atlas_transform_file)
        ])
    if atlas_scalefactor is not None:
        cargs.append(str(atlas_scalefactor))
    if etiv_transform_file is not None:
        cargs.extend([
            "-eTIV",
            execution.input_file(etiv_transform_file)
        ])
    if etiv_scalefactor is not None:
        cargs.append(str(etiv_scalefactor))
    if etiv_subject is not None:
        cargs.extend([
            "-eTIV_matdat",
            etiv_subject
        ])
    ret = MriLabelVolumeOutputs(
        root=execution.output_file("."),
        output_volume_results=execution.output_file("volume_label_results.txt"),
        output_log_results=execution.output_file("volume_log.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRI_LABEL_VOLUME_METADATA",
    "MriLabelVolumeOutputs",
    "mri_label_volume",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

FSLSTATS_METADATA = Metadata(
    id="c81a361a55a157923aa804164f6e93ae5f94ecdf.boutiques",
    name="fslstats",
    package="fsl",
    container_image_tag="brainlife/fsl:6.0.4-patched2",
)


class FslstatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `fslstats(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_stats: OutputPathType
    """Statistics output file"""


def fslstats(
    input_file: InputPathType,
    index_mask: InputPathType | None = None,
    lower_threshold: float | None = None,
    upper_threshold: float | None = None,
    robust_intensity_flag: bool = False,
    minmax_intensity_flag: bool = False,
    voxels_volume_flag: bool = False,
    nonzero_voxels_volume_flag: bool = False,
    mean_flag: bool = False,
    nonzero_mean_flag: bool = False,
    std_dev_flag: bool = False,
    nonzero_std_dev_flag: bool = False,
    smallest_roi_flag: bool = False,
    max_coords_flag: bool = False,
    min_coords_flag: bool = False,
    cog_mm_flag: bool = False,
    cog_voxel_flag: bool = False,
    percentile: float | None = None,
    nonzero_percentile: float | None = None,
    absolute_values_flag: bool = False,
    nan_as_zero_flag: bool = False,
    mask_image: InputPathType | None = None,
    difference_image: InputPathType | None = None,
    hist_bins: float | None = None,
    hist_bins_min_max: str | None = None,
    timeseries_flag: bool = False,
    mean_entropy_flag: bool = False,
    nonzero_mean_entropy_flag: bool = False,
    runner: Runner | None = None,
) -> FslstatsOutputs:
    """
    FSL tool for calculating statistics on image data.
    
    Author: FMRIB Analysis Group, University of Oxford
    
    URL: https://fsl.fmrib.ox.ac.uk/fsl/fslwiki
    
    Args:
        input_file: Input image file (e.g., image.nii.gz).
        index_mask: Generate separate n submasks from indexMask, for\
            indexvalues 1..n where n is the maximum index value in indexMask, and\
            generate statistics for each submask.
        lower_threshold: Set lower threshold.
        upper_threshold: Set upper threshold.
        robust_intensity_flag: Output robust min and max intensity.
        minmax_intensity_flag: Output min and max intensity.
        voxels_volume_flag: Output voxels and volume.
        nonzero_voxels_volume_flag: Output voxels and volume (for nonzero\
            voxels).
        mean_flag: Output mean.
        nonzero_mean_flag: Output mean (for nonzero voxels).
        std_dev_flag: Output standard deviation.
        nonzero_std_dev_flag: Output standard deviation (for nonzero voxels).
        smallest_roi_flag: Output smallest ROI containing nonzero voxels.
        max_coords_flag: Output coordinates of maximum voxel.
        min_coords_flag: Output coordinates of minimum voxel.
        cog_mm_flag: Output center-of-gravity (cog) in mm coordinates.
        cog_voxel_flag: Output center-of-gravity (cog) in voxel coordinates.
        percentile: Output nth percentile.
        nonzero_percentile: Output nth percentile (for nonzero voxels).
        absolute_values_flag: Use absolute values of all image intensities.
        nan_as_zero_flag: Treat NaN or Inf as zero for subsequent stats.
        mask_image: Use the specified image for masking - overrides lower and\
            upper thresholds.
        difference_image: Take the difference between the base image and the\
            image specified here.
        hist_bins: Output a histogram for the thresholded/masked voxels only\
            with specified number of bins.
        hist_bins_min_max: Output a histogram for the thresholded/masked voxels\
            only with specified number of bins and histogram limits of min and max.
        timeseries_flag: Separate output line for each 3D volume of a 4D\
            timeseries.
        mean_entropy_flag: Output mean entropy; mean(-i*ln(i)).
        nonzero_mean_entropy_flag: Output mean entropy (of nonzero voxels).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `FslstatsOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(FSLSTATS_METADATA)
    cargs = []
    cargs.append("fslstats")
    cargs.append(execution.input_file(input_file))
    if index_mask is not None:
        cargs.extend([
            "-k",
            execution.input_file(index_mask)
        ])
    if lower_threshold is not None:
        cargs.extend([
            "-l",
            str(lower_threshold)
        ])
    if upper_threshold is not None:
        cargs.extend([
            "-u",
            str(upper_threshold)
        ])
    if robust_intensity_flag:
        cargs.append("-r")
    if minmax_intensity_flag:
        cargs.append("-R")
    if voxels_volume_flag:
        cargs.append("-v")
    if nonzero_voxels_volume_flag:
        cargs.append("-V")
    if mean_flag:
        cargs.append("-m")
    if nonzero_mean_flag:
        cargs.append("-M")
    if std_dev_flag:
        cargs.append("-s")
    if nonzero_std_dev_flag:
        cargs.append("-S")
    if smallest_roi_flag:
        cargs.append("-w")
    if max_coords_flag:
        cargs.append("-x")
    if min_coords_flag:
        cargs.append("-X")
    if cog_mm_flag:
        cargs.append("-c")
    if cog_voxel_flag:
        cargs.append("-C")
    if percentile is not None:
        cargs.extend([
            "-p",
            str(percentile)
        ])
    if nonzero_percentile is not None:
        cargs.extend([
            "-P",
            str(nonzero_percentile)
        ])
    if absolute_values_flag:
        cargs.append("-a")
    if nan_as_zero_flag:
        cargs.append("-n")
    if mask_image is not None:
        cargs.extend([
            "-k",
            execution.input_file(mask_image)
        ])
    if difference_image is not None:
        cargs.extend([
            "-d",
            execution.input_file(difference_image)
        ])
    if hist_bins is not None:
        cargs.extend([
            "-h",
            str(hist_bins)
        ])
    if hist_bins_min_max is not None:
        cargs.extend([
            "-H",
            hist_bins_min_max
        ])
    if timeseries_flag:
        cargs.append("-t")
    if mean_entropy_flag:
        cargs.append("-e")
    if nonzero_mean_entropy_flag:
        cargs.append("-E")
    ret = FslstatsOutputs(
        root=execution.output_file("."),
        output_stats=execution.output_file(pathlib.Path(input_file).name + "_stats.txt"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "FSLSTATS_METADATA",
    "FslstatsOutputs",
    "fslstats",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MRIS_BA_SEGMENT_METADATA = Metadata(
    id="b3ec206494aa6dee2d2521daa7c2af2397b82e98.boutiques",
    name="mris_BA_segment",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MrisBaSegmentOutputs(typing.NamedTuple):
    """
    Output object returned when calling `mris_ba_segment(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_label_file: OutputPathType
    """Output label file"""


def mris_ba_segment(
    surface: InputPathType,
    profiles: InputPathType,
    prior_label: InputPathType,
    output_label: str,
    runner: Runner | None = None,
) -> MrisBaSegmentOutputs:
    """
    Segments a Brodmann area (MT currently) from a laminar intensity profile
    overlay.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        surface: Input surface file.
        profiles: Input profiles file.
        prior_label: Input prior label file.
        output_label: Output label file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MrisBaSegmentOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MRIS_BA_SEGMENT_METADATA)
    cargs = []
    cargs.append("mris_BA_segment")
    cargs.append(execution.input_file(surface))
    cargs.append(execution.input_file(profiles))
    cargs.append(execution.input_file(prior_label))
    cargs.append(output_label)
    ret = MrisBaSegmentOutputs(
        root=execution.output_file("."),
        output_label_file=execution.output_file(output_label),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MRIS_BA_SEGMENT_METADATA",
    "MrisBaSegmentOutputs",
    "mris_ba_segment",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

V__COMMAND_GLOBB_METADATA = Metadata(
    id="c2faf81a08e05a3e16060acc326e418fbdfeaad0.boutiques",
    name="@CommandGlobb",
    package="afni",
    container_image_tag="afni/afni_make_build:AFNI_24.2.06",
)


class VCommandGlobbOutputs(typing.NamedTuple):
    """
    Output object returned when calling `v__command_globb(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    output_files: OutputPathType
    """Output files generated by the specified program command line."""


def v__command_globb(
    program_command: str,
    output_dir: str,
    brick_list: list[str],
    extension: str | None = None,
    runner: Runner | None = None,
) -> VCommandGlobbOutputs:
    """
    A command-line tool to execute a specified program command line on a list of
    input bricks.
    
    Author: AFNI Developers
    
    URL: https://afni.nimh.nih.gov/
    
    Args:
        program_command: The entire command line for the program desired. The\
            command is best put between single quotes, do not use the \\ to break a\
            long line within the quotes.
        output_dir: The output directory where the results will be saved.
        brick_list: A list of bricks (or anything) on which the program command\
            will be executed.
        extension: If the program requires a -prefix option, then you can\
            specify the extension which will get appended to the Brick names before\
            +orig.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VCommandGlobbOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(V__COMMAND_GLOBB_METADATA)
    cargs = []
    cargs.append("@CommandGlobb")
    cargs.append("-com")
    cargs.append(program_command)
    cargs.append("-session")
    cargs.append(output_dir)
    cargs.append("-newxt")
    if extension is not None:
        cargs.append(extension)
    cargs.extend([
        "-list",
        *brick_list
    ])
    ret = VCommandGlobbOutputs(
        root=execution.output_file("."),
        output_files=execution.output_file(output_dir + "/*"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "VCommandGlobbOutputs",
    "V__COMMAND_GLOBB_METADATA",
    "v__command_globb",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

MAKE_AVERAGE_SUBJECT_METADATA = Metadata(
    id="d75a24ae99cd3258e4c0323f2c5c4fba80dcea8b.boutiques",
    name="make_average_subject",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class MakeAverageSubjectOutputs(typing.NamedTuple):
    """
    Output object returned when calling `make_average_subject(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def make_average_subject(
    subjects: list[str],
    average_subject_name: str,
    fsgd_file: InputPathType | None = None,
    subject_list_file: InputPathType | None = None,
    sd_out: str | None = None,
    no_link: bool = False,
    sdir: str | None = None,
    ico_order: float | None = 7,
    transform_file: InputPathType | None = None,
    surface_registration: str | None = "sphere.reg",
    no_surfaces: bool = False,
    no_volumes: bool = False,
    force: bool = False,
    keep_all_orig: bool = False,
    no_symlink: bool = False,
    no_ribbon: bool = False,
    no_surf2surf: bool = False,
    rca_threads: float | None = None,
    help_: bool = False,
    version: bool = False,
    echo: bool = False,
    debug: bool = False,
    runner: Runner | None = None,
) -> MakeAverageSubjectOutputs:
    """
    Creates an average subject by averaging surfaces, curvatures, and volumes from a
    set of subjects.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        subjects: List of subject names.
        average_subject_name: Name of the average subject.
        fsgd_file: Get subject list from a FreeSurfer Group Descriptor file.
        subject_list_file: Text file containing all subject names.
        sd_out: Directory to put output under instead of SUBJECTS_DIR.
        no_link: Do not link back to the original SUBJECTS_DIR with --sd-out.
        sdir: Use an alternative SUBJECTS_DIR instead of the default one in the\
            environment.
        ico_order: Change order of icosahedron.
        transform_file: Filename of transform file.
        surface_registration: Alternative registration surface name.
        no_surfaces: Do not make average surfaces.
        no_volumes: Do not make average volumes.
        force: Overwrite existing average subject data.
        keep_all_orig: Concatenate all original volumes into mri/orig.all.mgz.
        no_symlink: Do not use symbolic links with surfaces, copy files instead.
        no_ribbon: Do not create ribbon.mgz and aparc+aseg.mgz files.
        no_surf2surf: Use old parametric surface mapping method.
        rca_threads: Number of threads to pass to recon-all.
        help_: Show short descriptive help.
        version: Show script version info.
        echo: Enable command echo for debugging.
        debug: Enable debug mode, same as --echo.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MakeAverageSubjectOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(MAKE_AVERAGE_SUBJECT_METADATA)
    cargs = []
    cargs.append("make_average_subject")
    cargs.extend([
        "--subjects",
        *subjects
    ])
    if fsgd_file is not None:
        cargs.extend([
            "--fsgd",
            execution.input_file(fsgd_file)
        ])
    if subject_list_file is not None:
        cargs.extend([
            "--f",
            execution.input_file(subject_list_file)
        ])
    cargs.extend([
        "--out",
        average_subject_name
    ])
    if sd_out is not None:
        cargs.extend([
            "--sd-out",
            sd_out
        ])
    if no_link:
        cargs.append("--no-link")
    if sdir is not None:
        cargs.extend([
            "--sdir",
            sdir
        ])
    if ico_order is not None:
        cargs.extend([
            "--ico",
            str(ico_order)
        ])
    if transform_file is not None:
        cargs.extend([
            "--xform",
            execution.input_file(transform_file)
        ])
    if surface_registration is not None:
        cargs.extend([
            "--surf-reg",
            surface_registration
        ])
    if no_surfaces:
        cargs.append("--no-surf")
    if no_volumes:
        cargs.append("--no-vol")
    if force:
        cargs.append("--force")
    if keep_all_orig:
        cargs.append("--keep-all-orig")
    if no_symlink:
        cargs.append("--no-symlink")
    if no_ribbon:
        cargs.append("--no-ribbon")
    if no_surf2surf:
        cargs.append("--no-surf2surf")
    if rca_threads is not None:
        cargs.extend([
            "--rca-threads",
            str(rca_threads)
        ])
    if help_:
        cargs.append("--help")
    if version:
        cargs.append("--version")
    if echo:
        cargs.append("--echo")
    if debug:
        cargs.append("--debug")
    ret = MakeAverageSubjectOutputs(
        root=execution.output_file("."),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "MAKE_AVERAGE_SUBJECT_METADATA",
    "MakeAverageSubjectOutputs",
    "make_average_subject",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *
import dataclasses

LABEL_SUBJECT_METADATA = Metadata(
    id="91f9c5eb131863d4bbc7a95b5adc4b9fd63410f3.boutiques",
    name="label_subject",
    package="freesurfer",
    container_image_tag="freesurfer/freesurfer:7.4.1",
)


class LabelSubjectOutputs(typing.NamedTuple):
    """
    Output object returned when calling `label_subject(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    converted_mnc: OutputPathType
    """Converted MNC file from the original MRI data"""


def label_subject(
    nu_file: InputPathType | None = None,
    orig_dir: str | None = "/usr/local/freesurfer/subjects",
    runner: Runner | None = None,
) -> LabelSubjectOutputs:
    """
    A tool for labeling subject MRI data in FreeSurfer.
    
    Author: FreeSurfer Developers
    
    URL: https://github.com/freesurfer/freesurfer
    
    Args:
        nu_file: Path to the nu.mgz file. If not provided, defaults to nu.mgz.
        orig_dir: The original MRI data directory. If the nu file is not found,\
            it will search in the given directory.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelSubjectOutputs`).
    """
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABEL_SUBJECT_METADATA)
    cargs = []
    cargs.append("label_subject")
    if nu_file is not None:
        cargs.append(execution.input_file(nu_file))
    if orig_dir is not None:
        cargs.append(orig_dir)
    ret = LabelSubjectOutputs(
        root=execution.output_file("."),
        converted_mnc=execution.output_file("/tmp/nu*.mnc"),
    )
    execution.run(cargs)
    return ret


__all__ = [
    "LABEL_SUBJECT_METADATA",
    "LabelSubjectOutputs",
    "label_subject",
]
